unit juldat;

(*
    Protokoly 2.5
    Copyright (C) 2002  Ing. David Motl

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

interface

Uses SysUtils, Math;

Function  JDToDateTime(JD:double):TDateTime;
          // Prevede JD na datum a cas
Function  DateTimeToJD(DT:TDateTime):double;
          // Prevede datum a cas na JD
Function  JDToNightDate(JD:double):String;
          // Prevede JD na datum ve formatu  dd/dd-nn-yy
Function  JDToDateTimeStr(JD:double):String;
          // Prevede JD na datum ve formatu  dd-nn-yy hh:mm
Function  DateTimeToNightDate(DT:TDateTime):String;
          // Prevod DateTime na datum pozorovani dd/dd-nn-yy

Function  HelCorr(ARA,ADE,AJDgeo:double):double;
          { Vraci velikost heliocentricke korekce
            ARA - rektascenze v hodinach;
            ADE - deklinace ve stupnich;
            AJDgeo - geocentricke jd
            -----------------------------------
            JDHel = JDGeo + HelCorr
          }
Function  JDhel(JD,RA,DE:double):double;
          // Prevod na heliocentricke JD
Function  HvCas(jd,lon:double):double;
          // Mistni hvezdny cas
Procedure RaDeToRiseSet(Ra,De:double; jd0,jd,lon,lat:double; var cond:byte; var JDRise,JDtrans,JDSet:double);
          // Vychod a zapad objektu

implementation

Function JDToDateTime;
begin
 if JD<2000000 then JD:=JD+2400000;
 Result:=JD-2415018.5;
end;

Function DateTimeToJD;
begin
 Result:=2415018.5+DT;
end;

Function ArcSin(x:real):real;
{ Arkus sinus (radiany) }
begin
 ArcSin := ArcTan (x/sqrt (1-sqr (x)))
end;

Procedure Coords(ra,de:real; var la,be:real);
{ Prevede souradnice ra,de (rovnikove) na la, be (ekliptikalni) }
var eps:real;
begin
  eps:=(23+26/60)/180*pi;  { Sklon ekliptiky 2326' }
  { Prevod na radiany }
  ra:=ra/12*pi;
  de:=de/180*pi;
  while ra<0 do ra:=ra+2*pi;     { Do intervalu 0..2pi }
  while ra>=2*pi do ra:=ra-2*pi;
  { Vylouceni singularit }
  if de=pi/2 then begin
    la:=pi/2; be:=pi/2-eps;
  end else
  if de=-pi/2 then begin
    la:=3*pi/2; be:=eps-pi/2;
  end else
  if ra=pi/2 then begin
    la:=pi/2; be:=de-eps;
  end else
  if ra=3*pi/2 then begin
    la:=3*pi/2; be:=de+eps;
  end else begin
  { Normalni vypocet }
  be:=arcsin(cos(eps)*sin(de)-sin(eps)*cos(de)*sin(ra));
  la:=arctan((sin(eps)*sin(de)+cos(eps)*cos(de)*sin(ra))/(cos(de)*cos(ra)));
  if cos(ra)<0 then la:=la+pi; { Do spravneho kvadrantu }
  end;
end;

Procedure Sun(JD:real; var rs,las:real);
{ Vypocet ekl. delky slunce a vzdalenosti zeme - slunce }
var t,vt,Ls,Ms,M5,M4,M2,Lm:real;
begin
t:=jd-2451545;
vt:=1+(t/36525);
Ls:=(0.779072 + 0.00273790931*t)*2*pi;
ms:=(0.993126 + 0.00273777850*t)*2*pi;
M5:=(0.056531 + 0.00023080893*t)*2*pi;
M4:=(0.053856 + 0.00145561327*t)*2*pi;
M2:=(0.140023 + 0.00445036173*t)*2*pi;
Lm:=(0.606434 + 0.03660110129*t)*2*pi;
las:=Ls*3600*180/pi+6910*sin(ms)
      +  72*sin(2*ms)
      -17*vt*sin(ms)
      -   7*cos(ms-m5)
      +   6*sin(lm-ls)
      +   5*sin(4*ms+8*m4+3*m5)
      -   5*cos(2*ms-2*m2)
      -   4*sin(ms-m2)
      +   4*cos(4*ms-8*m4+3*m5)
      +   3*sin(2*ms-2*m2)
      -   3*sin(m5)
      -   3*sin(2*ms-2*m5);
las:=las/3600/180*pi;     { Ekl. delka slunce v radianech }
rs:=1.00014-0.01675*cos(ms)-0.00014*cos(2*ms); { vzdalenost slunce v AU }
end;

Function HelCorr;
var la, be     : real;      { Ekliptikalni souradnice objektu }
    rs, las    : real;      { Souradnice a vzdalenost slunce }
begin
 if ajdgeo<2000000 then ajdgeo:=ajdgeo+2400000;
 Coords(ara,ade,la,be);
 Sun(ajdgeo,rs,las);
 Result:=-0.00577552*rs*cos(be)*cos(la-las);
end;

Function JDhel;
begin
 Result:=JD+HelCorr(RA,DE,JD);
end;

Function JDToNightDate;
var DT:TDateTime; Y,M,D:word;
begin
 DT:=JDToDateTime(JD);
 if frac(JD)<0.5 { Pozorovani zacalo pred pulnoci } then begin
   DecodeDate(DT+1,Y,M,D);
 end else begin
   DecodeDate(DT,Y,M,D);
 end;
 Result:=Format('%2.2d/%2.2d-%2.2d-%2.2d',[D-1,D,M,(Y mod 100)]);
end;

Function JDToDateTimeStr;
var DT:TDateTime; H,M,S,MS,Y,N,D:word;
begin
 DT:=JDToDateTime(JD);
 DecodeTime(DT,H,N,S,MS);
 DecodeDate(DT,Y,M,D);
 Result:=Format('%2.2d-%2.2d-%2.2d %2.2d:%2.2d',[D,M,(Y mod 100),H,N]);
end;

Function HvCas;
var T3,S0:double;
begin
 T3:=(int(JD-0.5)+0.5-2451545)/36525;
 S0:=6.697374558+2400.05133691*T3+0.0000258622*T3*T3-0.0000000017*T3*T3*T3;
 Result:=S0+1.0027379093*(frac(jd)*24-12)+(lon/pi*12);
 while (Result<0) do Result:=Result+24; while (Result>=24) do Result:=Result-24;
end;

Procedure RaDeToRiseSet;
var T3,S0,x,y,TRise,TSet,TTrans:double;
begin
 JDRise:=0; JDSet:=0; cond:=0;
 if (lat=90) then begin if de<0 then cond:=1 else cond:=2; exit; end;
 if (lat=-90) then begin if de<0 then cond:=2 else cond:=1; exit; end;
 T3:=(int(JD-0.5)+0.5-2451545)/36525;
 S0:=(6.697374558+2400.05133691*T3+0.0000258622*T3*T3-0.0000000017*T3*T3*T3)/12*pi;
 while (s0<0) do s0:=s0+2*pi; while (s0>=2*pi) do s0:=s0-2*pi;
 x:=-tan(lat/180*pi)*tan(de);
 if x>=1 then begin cond:=1; exit; end
 else if x<=-1 then begin cond:=2; exit; end
 else begin
   TRise :=3.80929*(Ra-lon/180*pi-arccos(x)-S0);
   TTrans:=3.80929*(Ra-lon/180*pi          -S0);
   TSet  :=3.80929*(Ra-lon/180*pi+arccos(x)-S0);
   while (TSet<0) do TSet:=TSet+24; while (TSet>=24) do TSet:=TSet-24;
   while (TRise<0) do TRise:=TRise+24; while (TRise>=24) do TRise:=TRise-24;
   while (TTrans<0) do TTrans:=TTrans+24; while (TTrans>=24) do TTrans:=TTrans-24;
   JDRise :=int(JD-0.5)+TRise/24+0.5;
   JDTrans:=int(JD-0.5)+TTrans/24+0.5;
   JDSet  :=int(JD-0.5)+TSet/24+0.5;
   if (JDRise<JD0)  then RaDeToRiseSet(Ra,De,JD0,JD+0.5,Lon,Lat,cond,JDRise,x,y);
   if (JDTrans<JD0) then RaDeToRiseSet(Ra,De,JD0,JD+0.5,Lon,Lat,cond,x,JDTrans,y);
   if (JDSet<JD0)   then RaDeToRiseSet(Ra,De,JD0,JD+0.5,Lon,Lat,cond,x,y,JDSet);
 end;
end;

Function DateTimeToNightDate;
var R,M,D:word;
begin
 DecodeDate(DT+1,R,M,D);
 Result:=Format('%.2d./%.2d. %.2d. %.4d',[D-1,D,M,R]);
end;

end.

