unit Katalog;

(*
    Protokoly 2.1
    Copyright (C) 2002  Ing. David Motl

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

interface

Uses Forms, Classes, SysUtils, Math, Utils;

type
     TKatZaznam = class(TObject)
       RA, DEC, M0, PER : String;
       Constructor Create(Str:String='');
       Procedure   Assign(Str:String);
     end;

     TKatalog = class(TObject)
       Data:TStringList;
       MyDir, UserFile, TempFile:String;
       Constructor Create(AMyDir:String);
       Destructor  Destroy; override;
       Procedure   Update;
       Function    ConsList(L:TStrings; ASel:String):integer;
       Function    StarList(L:TStrings; ACons,ASel:String):integer;
       Procedure   LoadRec(AObj:String; var ARA,ADE,AM0,AP:String);
       Function    SaveRec(AObj:String; ARA,ADE,AM0,AP:String):boolean;
       Function    DelRec(AObj:String):boolean;
       Function    DatToStr(AObj:String; ARA,ADE,AM0,AP:String):String;
     end;

var Kat:TKatalog;

implementation

Constructor TKatZaznam.Create;
begin
 inherited Create;
 Assign(Str);
end;

(* BRKA2002:
 V 404 And     C12       9 010124.2 +411501  51924.3245  0.67601     11.3  11.9        Rc  BB 124;IBVS 4560                                                      02807.01423   *
   BRKA2004:
 MisV1095     13    10 002649.4 +414909  52572.50680 1.58255471   12.8  13.4  13.1  V  Prosper               3    0                                        02791.02148 N
   BRKA2005:
 MisV1095     13    10 002649.4 +414909 53225.322   1.58258351   12.8  13.4  13.1  V  Prosper               3    0                                        02791.02148 *
   PROSPER:
 MisV 1095    13    10 002650   +414908 2453224.531 1.58258351   12.8  13.4  13.1  V  vyp                   3    0.0  PROSPER                             02791.02148
   USER:
 GSC 27500854              230131   +304427  2452909.429 0.47164600
*)

Procedure TKatZaznam.Assign;
begin
 if (copy(Str,34,1)='.')or(copy(Str,34,1)=' ') then begin
   { BRKA 2002, USER }
   RA:=trim(copy(Str,28,8));
   DEC:=trim(copy(Str,37,7));
   M0:=trim(copy(Str,46,11));
   PER:=trim(copy(Str,58,11));
 end else
 if (copy(Str,41,1)=' ') then begin
   { BRKA 2004 }
   RA:=trim(copy(Str,24,8));
   DEC:=trim(copy(Str,33,7));
   M0:=trim(copy(Str,42,11));
   PER:=trim(copy(Str,54,11));
 end else begin
   { BRKA 2005, PROSPER }
   RA:=trim(copy(Str,24,8));
   DEC:=trim(copy(Str,33,7));
   M0:=trim(copy(Str,41,11));
   PER:=trim(copy(Str,53,12));
 end;
 while (pos('.',M0)>0) do M0[pos('.',M0)]:=',';
 while (pos('.',PER)>0) do PER[pos('.',PER)]:=',';
end;

Function TKatalog.DatToStr;
begin
 { Rektascenze }
 if Length(ARA)<6 then
   raise Exception.Create('Chyba v zpisu rektascenze.');
 { Deklinace }
 if Length(ADE)<7 then
   raise Exception.Create('Chyba v zpisu deklinace.');
 { Zakladni minimum }
 while (pos('.',AM0)>0) do AM0[pos('.',AM0)]:=',';
 if StrToFloatDef(AM0,-1)<0 then
   raise Exception.Create('Chyba v zpisu zkladnho minima.');
 while (pos(',',AM0)>0) do AM0[pos(',',AM0)]:='.';
 { Perioda }
 while (pos('.',AP)>0) do AP[pos('.',AP)]:=',';
 if StrToFloatDef(AP,0)<=0 then
   raise Exception.Create('Chyba v zpisu periody.');
 while (pos(',',AP)>0) do AP[pos(',',AP)]:='.';
 Result:=Format(' %-13s             %-8s %-8s %-11s %-11s',[AObj,ARA,ADE,AM0,AP]);
end;

Constructor TKatalog.Create;
begin
 inherited Create;
 MyDir:=AMyDir;
// BrkaFile:=MyDir+'\brka.dat';
 UserFile:=MyDir+'\user.dat';
 TempFile:=MyDir+'\user.$$$';
 Data:=TStringList.Create;
end;

Destructor TKatalog.Destroy;
var i:integer;
begin
 with Data do for i:=0 to Count-1 do Objects[i].Free;
 Data.Free;
 inherited;
end;

Procedure TKatalog.Update;
var F:TextFile;
    obj,line:String;
    Res,i:integer;
    SR:TSearchRec;
begin
 { Vymazat stavajici data }
 with Data do for i:=0 to Count-1 do Objects[i].Free;
 Data.Clear;
 try
   Res:=FindFirst(MyDir+'\*.dat',faAnyFile,SR);
   while (Res=0) do begin
     try
       AssignFile(F,MyDir+'\'+SR.Name); Reset(F);
       try
         while not eof(f) do begin
           readln(F,line);
           if (copy(line,1,1)=' ')and(copy(line,1,6)<>' NAZEV') then begin
             obj:=trim(copy(line,2,13));
             i:=Data.IndexOf(obj);
             if i>=0 then TKatZaznam(Data.Objects[i]).Assign(line)
                     else Data.AddObject(obj,TKatZaznam.Create(line));
           end;
         end;
       finally
         CloseFile(f);
       end;
     except
     end;
     Res:=FindNext(SR);
   end;
 finally
   FindClose(SR);
 end;
end;

Procedure TKatalog.LoadRec;
var Index:integer;
begin
 Index:=Data.IndexOf(AObj);
 if Index>=0 then with TKatZaznam(Data.Objects[Index]) do begin
   ARA:=RA;
   ADE:=DEC;
   AM0:=M0;
   AP :=PER;
 end else begin
   ARA:='000000.0'; ADE:='+000000'; AM0:='52000,0'; AP:='1,5';
 end;
end;

Function TKatalog.SaveRec;
var Index:integer; F,G:TextFile; ok:boolean; line,obj:String;
    Z:TKatZaznam;
begin
 { Ulozit do pameti }
 Index:=Data.IndexOf(AObj);
 if Index>=0 then with TKatZaznam(Data.Objects[Index]) do begin
   RA :=ARA;
   DEC:=ADE;
   M0 :=AM0;
   PER:=AP;
 end else begin
   Z:=TKatZaznam.Create;
   Z.RA :=ARA;
   Z.DEC:=ADE;
   Z.M0 :=AM0;
   Z.PER:=AP;
 end;
 { Ulozit do souboru USER.DAT }
 if FileExists(UserFile) then begin
   { Nacist data z USER.DAT }
   AssignFile(F,UserFile);   reset(F);
   AssignFile(G,TempFile);   rewrite(G);
   try
     try
       ok:=false;
       while not eof(f) do begin
         readln(F,line);
         obj:=trim(copy(line,2,13));
         if Uppercase(obj)=Uppercase(AObj) then begin
           { Nasel jsem radek, ktery se musi opravit }
           Writeln(G,DatToStr(AObj,ARA,ADE,AM0,AP));
           ok:=true; { Uz jsem zapsal }
         end else begin
           { Radek zustava }
           Writeln(G,line);
         end;
       end;
       if not ok then
         { Novy zaznam nakonec }
         Writeln(G,DatToStr(AObj,ARA,ADE,AM0,AP));
     finally
       CloseFile(f);
       CloseFile(g);
     end;
     DeleteFile(UserFile);
     RenameFile(TempFile,UserFile);
     Result:=true;
   except
     DeleteFile(TempFile);
     Result:=false;
   end;
 end else begin
   { User.dat jeste neexistuje }
   try
     AssignFile(G,UserFile);   rewrite(G);
     Writeln(G,' NAZEV                     RA2000   DE2000   M0          PER');
     Writeln(G,DatToStr(AObj,ARA,ADE,AM0,AP));
     CloseFile(g);
     Result:=true;
   except
     Result:=false;
   end;
 end;
end;

Function TKatalog.DelRec;
var Index:integer; F,G:TextFile; ok:boolean; line,obj:String;
begin
 Result:=false;
 { Vymazat ze souboru USER.DAT }
 if FileExists(UserFile) then begin
   { Nacist data z USER.DAT }
   AssignFile(F,UserFile);   reset(F);
   AssignFile(G,TempFile);   rewrite(G);
   try
     try
       ok:=false;
       while not eof(f) do begin
         readln(F,line);
         obj:=trim(copy(line,2,13));
         if Uppercase(obj)=Uppercase(AObj) then begin
           { Nasel jsem radek, ktery se musi vymazat }
           ok:=true; { Uz jsem vymazal }
         end else begin
           { Radek zustava }
           Writeln(G,line);
         end;
       end;
     finally
       CloseFile(f);
       CloseFile(g);
     end;
   except
     DeleteFile(TempFile);
     raise;
   end;
   DeleteFile(UserFile);
   RenameFile(TempFile,UserFile);
   if ok then begin
     { Vymazat z pameti }
     Index:=Data.IndexOf(AObj);
     if Index>=0 then Data.Delete(Index);
     Result:=true;
   end;
 end;
end;

Function TKatalog.ConsList;
var i,Index:integer; C,C1,S:String;
begin
 Result:=-1;
 if (Length(ASel)>=5)and((ASel[Length(ASel)-3]=' ')or(ASel[Length(ASel)-3]=':')) then
   C:=copy(ASel,Length(ASel)-2,3)
 else
   C:='';
 L.BeginUpdate;
 L.Clear;
 try
   for i:=0 to Data.Count-1 do if Length(Data[i])>=5 then begin
     S:=Data[i];
     if (S[Length(S)-3]=' ')or(S[Length(S)-3]=':') then begin
       C1:=copy(S,Length(S)-2,3);
       if L.IndexOf(C1)<0 then begin
         Index:=L.Add(C1);
         if (Length(C)>0)and(C1=C) then Result:=Index;
       end;
     end;
   end;
   Index:=L.Add('OSTATN');
   if (Length(ASel)>0)and(Result<0) then Result:=Index
   else if Result<0 then Result:=0;
 finally
   L.EndUpdate;
 end;
end;

Function TKatalog.StarList;
var i,Index:integer; C1,J1,S,J:String; Sek,S1:boolean;
begin
 Result:=-1;
 L.BeginUpdate;
 L.Clear;
 Sek:=pos(':',ASel)>0;
 if (Length(ASel)>=5)and((ASel[Length(ASel)-3]=' ')or(ASel[Length(ASel)-3]=':')) then
   J:=DelWS(copy(ASel,1,Length(ASel)-4))
 else
   J:=DelWS(ASel);
 try
   for i:=0 to Data.Count-1 do begin
     S:=Data[i];
     if (Length(Data[i])>=5) and ((S[Length(S)-3]=' ')or(S[Length(S)-3]=':')) then begin
       { Jmena hvezd, kde je uvedeno souhvezdi }
       C1:=copy(S,Length(S)-2,3);
       J1:=DelWS(copy(S,1,Length(S)-4));
       S1:=pos(':',S)>0;
       if (Length(ACons)=3)and(C1=ACons) then begin
         Index:=L.Add(S);
         if (J=J1)and(S1=Sek) then Result:=Index;
       end;
     end else begin
       { Jmena hvezd, ve kterych neni souhvezdi }
       if (Length(ACons)<>3) then begin
         S1:=pos(':',S)>0;
         J1:=DelWS(S);
         Index:=L.Add(S);
         if (J1=ASel)and(S1=Sek) then Result:=Index;
       end;
     end;
   end;
 finally
   L.EndUpdate;
 end;
end;

initialization
begin
 Kat:=NIL;
end;

finalization
begin
 Kat.Free;
end;

end.
