unit Konfig;

(*
    Protokoly 2.1
    Copyright (C) 2002  Ing. David Motl

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

interface

Uses Classes, Controls, StdCtrls, Forms, SysUtils, XMLStream, Math, TypClass;

const
  { Konfigurace }
  KonfigSoubor  : String = 'protokoly.cfg';
  MaxHistorySize = 10;

  dkKonfigTag   = 'KONFIG';
  dkDataPath    = 'P_CESTA';
  dkImPath      = 'I_CESTA';
  dkExPath      = 'E_CESTA';
  dkAktivni     = 'AKTIVNI';

type
  TKonfig = class(TObject)
   private
    Element:String;
    Aktivni:boolean;
   public
    KonfigFileName :String;      { Jmeno konfiguracniho souboru }
    DataPath:String;             { Adresar k ulozenym souborum }
    ImPath :String;              { Adresar k importovanym souborum }
    ExPath :String;              { Adresar k exportovanym souborum }
    LastTyp:String;
    Procedure UpdateHistory;
              { Aktualizuje historie u vyberovych poli }
    Procedure LoadKonfig;
    Procedure SaveKonfig;
    Procedure KonfigStartTag(ASender:TXMLParser; AnElement:String; S:TStrings);
    Procedure KonfigPCData(ASender:TXMLParser; Data:String);
  end;

var
  Konf : TKonfig;

implementation

Uses Main;

Procedure TKonfig.LoadKonfig;
{ Nacteni konfigurace ve formatu XML }
var P:TXMLParser;
begin
 with MainForm do begin
   Pozorovatel.Items.Clear;   Pozorovatel.Tag:=0;
   Bydliste.Items.Clear;      Bydliste.Tag:=0;
   Stanoviste.Items.Clear;    Stanoviste.Tag:=0;
   Pristroj.Items.Clear;      Pristroj.Tag:=0;
   Kamera.Items.Clear;        Kamera.Tag:=0;
   Filtr.Items.Clear;         Filtr.Tag:=0;
   Zvetseni.Items.Clear;      Zvetseni.Tag:=0;
   Podminky.Items.Clear;      Podminky.Tag:=0;
   Mapka.Items.Clear;         Mapka.Tag:=0;
   LastTyp:='';
   if FileExists(KonfigFileName) then begin
     IsUpdating:=true;
     Screen.Cursor:=crHourGlass;
     P:=TXMLParser.Create(TFileStream.Create(KonfigFileName,fmOpenRead));
     try
       P.OnStartTag:=KonfigStartTag;
       P.OnPCData:=KonfigPCData;
       P.Parse;
     finally
       IsUpdating:=false;
       P.Free;
       Screen.Cursor:=crDefault;
     end;
   end;
   if Pozorovatel.Items.Count>0 then Pozorovatel.ItemIndex:=Pozorovatel.Tag;
   if Bydliste.Items.Count>0 then Bydliste.ItemIndex:=Bydliste.Tag;
   if Stanoviste.Items.Count>0 then Stanoviste.ItemIndex:=Stanoviste.Tag;
   if Pristroj.Items.Count>0 then Pristroj.ItemIndex:=Pristroj.Tag;
   if Kamera.Items.Count>0   then Kamera.ItemIndex:=Kamera.Tag;
   if Filtr.Items.Count>0    then Filtr.ItemIndex:=Filtr.Tag;
   if Zvetseni.Items.Count>0 then Zvetseni.ItemIndex:=Zvetseni.Tag;
   if Podminky.Items.Count>0 then Podminky.ItemIndex:=Podminky.Tag;
   if Length(DataPath)=0     then DataPath:=ExtractFileDir(Application.EXEName);
   if Length(ImPath)=0       then ImPath:='C:\';
   if Length(ExPath)=0       then ExPath:='C:\';
 end;
end;

Procedure TKonfig.SaveKonfig;
{ Ulozeni konfigurace ve formatu XML }
var F:TStream;

  Procedure SaveHistory(ATag:String; ItemIndex:integer; A:TStrings);
  { Ulozeni tabulky historie }
  var i:integer;
  begin
   for i:=0 to min(MaxHistorySize,A.Count)-1 do begin
     if i=ItemIndex then begin
       XMLBeginTag(F,ATag);
       XMLAttribute(F,dkAktivni,TRUE);
       XMLDoneTag(F,false);
       XMLPCLine(F,A[i]);
       XMLEndTag(F,ATag);
     end else begin
       XMLStartTag(F,ATag,false);
       XMLPCLine(F,A[i]);
       XMLEndTag(F,ATag);
     end;
   end;
  end;

begin
 F:=TFileStream.Create(KonfigFileName,fmCreate);
 Screen.Cursor:=crHourGlass;
 try
   XMLHeader(F);
   XMLStartTag(F,dkKonfigTag,false);
   with MainForm do begin
     { Hlavicka }
     SaveHistory(dkPozorovatel,Pozorovatel.ItemIndex,Pozorovatel.Items);
     SaveHistory(dkBydliste,Bydliste.ItemIndex,Bydliste.Items);
     SaveHistory(dkStanoviste,Stanoviste.ItemIndex,Stanoviste.Items);
     SaveHistory(dkPristroj,Pristroj.ItemIndex,Pristroj.Items);
     SaveHistory(dkKamera,Kamera.ItemIndex,Kamera.Items);
     SaveHistory(dkFiltr,Filtr.ItemIndex,Filtr.Items);
     SaveHistory(dkZvetseni,Zvetseni.ItemIndex,Zvetseni.Items);
     SaveHistory(dkPodminky,Podminky.ItemIndex,Podminky.Items);
     SaveHistory(dkMapka,Mapka.ItemIndex,Mapka.Items);
     XMLSimpleData(F,dkTyp,LastTyp);
     XMLSimpleData(F,dkDataPath,DataPath);
     XMLSimpleData(F,dkImPath,ImPath);
     XMLSimpleData(F,dkExPath,ExPath);
   end;
   XMLEndTag(F,dkKonfigTag);
 finally
   Screen.Cursor:=crDefault;
   F.Free;
 end;
end;

Procedure TKonfig.KonfigStartTag;
{ Veskere informace jsou jako PCDATA }
begin
 Element:=AnElement;
 Aktivni:=XMLBoolean(S,dkAktivni,false);
end;

Procedure TKonfig.KonfigPCData;
{ Nacti data, podle toho, ktery byl posledni element }

  procedure LoadHistory(C:TComboBox; S:String);
  var Index:integer;
  begin
    Index:=C.Items.Add(S);
    if Aktivni then C.Tag:=Index;
  end;

begin
 with MainForm do begin
 if Element=dkPozorovatel then LoadHistory(Pozorovatel,Data) else
 if Element=dkBydliste    then LoadHistory(Bydliste,Data) else
 if Element=dkStanoviste  then LoadHistory(Stanoviste,Data) else
 if Element=dkPristroj    then LoadHistory(Pristroj,Data) else
 if Element=dkKamera      then LoadHistory(Kamera,Data) else
 if Element=dkFiltr       then LoadHistory(Filtr,Data) else
 if Element=dkZvetseni    then LoadHistory(Zvetseni,Data) else
 if Element=dkPodminky    then LoadHistory(Podminky,Data) else
 if Element=dkMapka       then LoadHistory(Mapka,Data) else
 if Element=dkTyp         then LastTyp:=Data else
 if Element=dkDataPath    then DataPath:=Data else
 if Element=dkImPath      then ImPath:=Data else
 if Element=dkExPath      then ExPath:=Data;
end;
end;

Procedure TKonfig.UpdateHistory;
{ Aktualizace historii vyberovych policek }

 Procedure Update(A:TComboBox);
 { Pridej do historie novy zaznam, pokud tam jeste neni. Omezeni poctu polozek }
 var ok:boolean; i:integer;
 begin
  if Length(A.Text)>0 then begin
    ok:=true;
    { Porovna, jestli uz tam neni }
    for i:=0 to A.Items.Count-1 do
      if A.Items[i]=A.Text then begin ok:=false; break; end;
    if ok then begin
      { Vymaze prebytecne }
      if A.ItemIndex=-1 then begin
        while A.Items.Count>=MaxHistorySize do A.Items.Delete(A.Items.Count-1);
      end;
      { Vlozi novy zaznam }
      A.Items.Insert(0,A.Text);
    end;
  end;
 end;

begin
 { Aktualizuje seznam historie u editacnich policek }
 Update(MainForm.Pozorovatel);
 Update(MainForm.Bydliste);
 Update(MainForm.Stanoviste);
 Update(MainForm.Pristroj);
 Update(MainForm.Zvetseni);
 Update(MainForm.Kamera);
 Update(MainForm.Filtr);
 Update(MainForm.Podminky);
 Update(MainForm.Mapka);
end;

initialization
begin
 Konf:=TKonfig.Create;
end;

finalization
begin
 Konf.Free;
end;

end.
