unit TypCCD;

(*
    Protokoly 2.4
    Copyright (C) 2002  Ing. David Motl

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

interface

Uses Controls, Forms, Classes, Graphics, SysUtils, XMLStream, TypClass, Math,
     Juldat, Grids, StdCtrls, Utils, Dialogs, Tisk;

type
  TFormularCCD = class(TFormular)
   protected
     TabStart,TabStop:integer;
     Procedure TiskZahlavi(P:TPrintEngine);
     Procedure TiskZapati(P:TPrintEngine);
     Procedure TiskZahlaviTabulky(P:TPrintEngine);
     Procedure TiskGrafu(P:TPrintEngine);
   public
     Function  GetTime(ARow:integer; out Value:double):boolean; override;
               { Vraci cas odhadu jako jd }
     Function  GetValue(ARow:integer; out Value:double; out Mode:byte):boolean; override;
               { Vraci hvezdnou velikost odhadu }
     Procedure DataStartTag(ASender:TXMLParser; AnElement:String; S:TStrings); override;
     Procedure DataPCData(ASender:TXMLParser; Data:String); override;
     Procedure ClearData; override;
     Procedure ClearGrid; override;
     Procedure SaveData(F:TStream); override;
     Procedure GraphPaint; override;
     Procedure ImportData(AFileName:String); override;
     Procedure ExportData(AFormat:integer; AFileName:String); override;
     Procedure UpdateGrid(ACol,ARow:integer); override;
     Procedure RefreshRow(ARow:integer); override;
     Procedure RefreshData; override;
     Procedure GotoNext; override;
     Procedure Print(P:TPrintEngine); override;
  end;

implementation

Uses Main;

Procedure TFormularCCD.DataStartTag;
{ Zpracovani tagu pri cteni proejktu }
var d,r:integer; Yr,Mo,Da,h,m:word; DT:TDateTime;
begin
 Element:=AnElement;
 if Element=dkHvezda then with MainForm do begin
   { Hvezda }
   Hvezda.Text:=XMLString(S,dkOznaceni,'');
   RA.Text :=CheckDec(XMLString(S,dkRA,''));
   DEC.Text:=CheckDec(XMLString(S,dkDEC,''));
   MIN.Text:=CheckDec(XMLString(S,dkM0,''));
   PER.Text:=CheckDec(XMLString(S,dkPER,''));
 end else
 if Element=dkPristroj then with MainForm do begin
   { Pristroj }
   Pristroj.Text:=XMLString(S,dkOznaceni,'');
   Kamera.Text:=XMLString(S,dkKamera,'');
   Filtr.Text:=XMLString(S,dkFiltr,'');
 end else
 if Element=dkPozorovatel then with MainForm do begin
   { Pozorovatel }
   Pozorovatel.Text:=XMLString(S,dkJmeno,'');
   Bydliste.Text:=XMLString(S,dkBydliste,'');
 end else
 if Element=dkDatum then with MainForm do begin
   { Datum pozorovani }
   D:=XMLInteger(S,dkDen2,1);
   M:=XMLInteger(S,dkMesic,1);
   R:=XMLInteger(S,dkRok,2000);
   Datum.Date:=encodeDate(R,M,D)-1;
 end else
 if Element=dkData then with MainForm.Grid do begin
   { Data }
   RowCount:=max(RowCount,GridIndex+1);
   Cells[0,GridIndex]:=IntToStr(GridIndex);
   Cells[1,GridIndex]:=CheckDec(XMLString(S,dkJd,''));
   Cells[2,GridIndex]:=CheckDec(XMLString(S,dkDMag,''));
   Cells[3,GridIndex]:=CheckDec(XMLString(S,dkFaze,''));
   Cells[4,GridIndex]:=XMLString(S,dkPoznamka,'');
   GridIndex:=GridIndex+1;
 end else
 if Element=dkMinimum then with MainForm do begin
   { Okamzik minima - muze byt uveden datum a cas, jd nebo oboje }
   try
     if S.IndexOfName(dkJD)>=0 then begin
       { Je uvedeno jd }
       JDMin:=StrToFloatDef(CheckDec(XMLString(S,dkJD,'')),0);
       if (JDMin<30000) then begin
         MessageDlg('V souboru je uloen chybn okamik minima.',mtWarning,[mbOK],0);
         JDMin:=int(DateTimeToJD(Datum.Date))+0.5;
       end else
       if JDMin<2000000 then begin
         JDMin:=JDMin+2400000;
       end;
     end else begin
       { Je uveden datum a cas }
       H:=XMLInteger(S,dkHodina,0);
       M:=XMLInteger(S,dkMinuta,0);
       DecodeDate(Datum.Date,Yr,Mo,Da);
       Yr:=XMLInteger(S,dkRok,Yr);
       Mo:=XMLInteger(S,dkMesic,Mo);
       Da:=XMLInteger(S,dkDen,Da);
       if H>12 then DT:=EncodeDate(Yr,Mo,Da)+EncodeTime(H,M,0,0)
               else DT:=EncodeDate(Yr,Mo,Da)+EncodeTime(H,M,0,0)+1;
       JDMin:=DateTimeToJD(DT);
       if JDMin<2000000 then JDMin:=JDMin+2400000;
     end;
   except
     JDMin:=2452000.5;
   end;
 end else
 if Element=dkOdhadu then with MainForm do begin
   { Pocet odhadu }
   Pocet.Text     :=XMLString(S,dkCelkem,'');
   Sestupnych.Text:=XMLString(S,dkSestupnych,'');
 end;
end;

Procedure TFormularCCD.DataPCData;
{ Nacti data, podle toho, jaky byl naposledy element }
var Index:integer;

  Procedure Load(C:TComboBox; Data:String);
  begin
   Index:=C.Items.IndexOf(Data);
   if Index>=0 then C.ItemIndex:=Index else C.Text:=Data;
  end;

begin
 with MainForm do begin
   if Element=dkStanoviste then Load(Stanoviste,Data) else
   if Element=dkPodminky   then Load(Podminky,Data);
 end;
end;

Procedure TFormularCCD.ClearData;
{ Inicializace celeho okna pri otevreni projektu }
begin
 inherited;
 with MainForm do begin
   CompSheet.TabVisible:=false;
   Kamera.Visible:=true;
   Filtr.Visible :=true;
   Zvetseni.Visible:=false;
   ToolButton8.Visible:=true;
   ToolButton12.Visible:=true;
   Mapka.Visible:=false;
   Katalog.Visible:=true;
 end;
 ClearGrid;
end;

Procedure TFormularCCD.ClearGrid;
{ Vymazani dat z tabulek }
var i,j:integer;
begin
 with MainForm, Grid do begin
   DefaultRowHeight:=Canvas.TextHeight('')+4;
   ColCount:=5;
   RowCount:=2;
   { Nastaveni sloupcu }
   Cells[0,0]:='slo';     ColWidths[0]:=40;
   Cells[1,0]:='JDgeo';     ColWidths[1]:=100;
   Cells[2,0]:='Dmag';      ColWidths[2]:=120;
   Cells[3,0]:='Fze';      ColWidths[3]:=120;
   Cells[4,0]:='Poznmka';  ColWidths[4]:=Width-290-120;
   { Nastaveni radku }
   for i:=1 to RowCount-1 do Cells[0,i]:=IntToStr(i);
   { Vymazani dat }
   for i:=1 to ColCount-1 do for j:=1 to RowCount-1 do Cells[i,j]:='';
 end;
 UpdateGrid(MainForm.Grid.Col,MainForm.Grid.Row);
end;

Procedure TFormularCCD.SaveData;
{ Ulozeni projektu ve formatu XML }
var i:integer; r,m,d,h,mi,s,ms:word; jd,dt:double;
begin
 XMLHeader(F);
 XMLBeginTag(F,dkProjektTag);
 with MainForm do begin
   { Hlavicka }
   XMLAttribute(F,dkTyp,dvTypCCD);
   XMLAttribute(F,dkVerze,Verze0);
   XMLDoneTag(F,false);
   { Hvezda }
   XMLBeginTag(F,dkHvezda);
   XMLAttribute(F,dkOznaceni,Hvezda.Text);
   XMLAttribute(F,dkRA,SetDec(RA.Text));
   XMLAttribute(F,dkDEC,SetDec(DEC.Text));
   XMLAttribute(F,dkM0,SetDec(Min.Text));
   XMLAttribute(F,dkPER,SetDec(Per.Text));
   XMLDoneTag(F,true);
   { Pozorovatel }
   XMLBeginTag(F,dkPozorovatel);
   XMLAttribute(F,dkJmeno,Pozorovatel.Text);
   XMLAttribute(F,dkBydliste,Bydliste.Text);
   XMLDoneTag(F,true);
   { Stanoviste }
   XMLSimpleData(F,dkStanoviste,Stanoviste.Text);
   { Pristroj }
   XMLBeginTag (F,dkPristroj);
   XMLAttribute(F,dkOznaceni,Pristroj.Text);
   XMLAttribute(F,dkKamera,Kamera.Text);
   XMLAttribute(F,dkFiltr,Filtr.Text);
   XMLDoneTag(F,true);
   { Datum pozorovani }
   try
     JD:=DateTimeToJD(Datum.Date+1);
     DecodeDate(Datum.Date+1,R,M,D);
     XMLBeginTag  (F,dkDatum);
     XMLAttribute(F,dkDen1,D-1);
     XMLAttribute(F,dkDen2,D);
     XMLAttribute(F,dkMesic,M);
     XMLAttribute(F,dkRok,R);
     XMLAttribute(F,dkJD,SetDec(Format('%.1f',[JD])));
     XMLDoneTag(F,true);
   except
   end;
   { Podminky }
   XMLSimpleData(F,dkPodminky,Podminky.Text);
   { Data }
   with Grid do begin
     for i:=1 to RowCount-1 do if (Length(Cells[1,i])>0) and (Length(Cells[2,i])>0) then begin
       XMLBeginTag(F,dkData);
       XMLAttribute(F,dkJd   ,SetDec(Cells[1,i]));
       XMLAttribute(F,dkDMag ,SetDec(Cells[2,i]));
       XMLAttribute(F,dkFaze ,SetDec(Cells[3,i]));
       if Length(Cells[4,i])>0 then XMLAttribute(F,dkPoznamka,Cells[4,i]);
       XMLDoneTag(F,true);
     end;
   end;
   { Okamzik minima }
   try
     XMLBeginTag(F,dkMinimum);
     XMLAttribute(F,dkJD,SetDec(Format('%.4f',[JDMin])));
     XMLAttribute(F,dkHelKor,SetDec(Format('%.4f',[HelKor])));
     XMLAttribute(F,dkJDHel,SetDec(Format('%.4f',[JDHel])));
     DT:=JDToDateTime(JDMin); DecodeDate(DT,R,M,D); DecodeTime(DT,H,Mi,S,MS);
     XMLAttribute(F,dkRok,R);
     XMLAttribute(F,dkMesic,M);
     XMLAttribute(F,dkDen,D);
     XMLAttribute(F,dkHodina,H);
     XMLAttribute(F,dkMinuta,Mi);
     XMLDoneTag(F,true);
   except
   end;
   { Pocet odhadu }
   XMLBeginTag(F,dkOdhadu);
   XMLAttribute(F,dkCelkem,Pocet.Text);
   XMLAttribute(F,dkSestupnych,Sestupnych.Text);
   XMLDoneTag(F,true);
 end;
 XMLEndTag(F,dkProjektTag);
end;

Procedure TFormularCCD.ImportData;
var F:TextFile;
    Row,x:integer;
    str,line:String;
    dt,gjd,mag:double;
    First:boolean;
begin
 ClearGrid;
 Row:=1; First:=true;
 AssignFile(F,AFileName); reset(f);
 try
   while not eof(f) do begin
     readln(f,line);
     x:=pos(' ',line);
     if x=0 then continue;
     str:=copy(line,1,x-1); line:=copy(line,x+1,Length(Line)-x);
     gjd:=StrToFloatDef(CheckDec(str),-1);
     x:=pos(' ',line);
     if x=0 then begin
       str:=line;
     end else begin
       str:=copy(line,1,x-1); line:=copy(line,x+1,Length(Line)-x);
     end;
     mag:=StrToFloatDef(CheckDec(str),-1);
     { Kontrola rozsahu hodnot }
     if (gjd>2000000) then gjd:=gjd-2400000;
     if (mag<-99)or(mag>99)or(gjd<=0) then continue;
     { Ulozit do radku tabulky }
     if Row>1 then MainForm.Grid.RowCount:=Row+1;
     MainForm.Grid.Cells[0,Row]:=IntToStr(Row);
     MainForm.Grid.Cells[1,Row]:=Format('%.4f',[gjd]);
     MainForm.Grid.Cells[2,Row]:=Format('%.3f',[mag]);
     MainForm.Grid.Cells[3,Row]:='';
     MainForm.Grid.Cells[4,Row]:='';
     { Musi se urcit datum a cas zacatku pozorovani }
     if First then begin
       First:=false;
       DT:=JDToDateTime(gjd);
       if frac(gjd)>=0.5 then MainForm.Datum.Date:=int(DT-1)
                         else MainForm.Datum.Date:=int(dt);
       MainForm.JDMin:=gjd+2400000;
     end;
     { Dalsi radek }
     Row:=Row+1;
   end;
 finally
   closeFile(f);
 end;
end;

Procedure TFormularCCD.ExportData;
var F:TextFile; R:integer; jd,v:double; line:String; m:byte;
begin
 AssignFile(F,AFileName); rewrite(f);
 try
   with MainForm.Grid do for R:=1 to RowCount-1 do begin
     if GetTime(R, jd) and GetValue(R, v, m) then begin
       if AFormat=1 then jd:=frac(jd);
       line:=Format('%.5f %.3f',[jd,v]);
       if AFormat<>2 then line:=SetDec(line);
       Writeln(f,line);
     end;
   end;
 finally
   closeFile(f);
 end;
end;

Function TFormularCCD.GetTime;
begin
 Result:=TryStrToFloat(CheckDec(MainForm.Grid.Cells[1,ARow]),Value);
 if Result and (Value>0) and (Value<2000000) then Value:=Value+2400000;
end;

Function TFormularCCD.GetValue;
var Aux:String;
begin
 Aux:=CheckDec(MainForm.Grid.Cells[2,ARow]);
 Mode:=0;
 if copy(Aux,1,1)='<' then begin
   Delete(Aux,1,1);
   Mode:=1;
 end else
 if copy(Aux,1,1)='>' then begin
   Delete(Aux,1,1);
   Mode:=2;
 end else
 if (Length(Aux)>0) and (copy(Aux,Length(Aux),1)=':') then begin
   Delete(Aux,Length(Aux),1);
   Mode:=3;
 end;
 Result:=TryStrToFloat(Aux,Value);
end;

procedure TFormularCCD.GraphPaint;
{ Vykresleni grafu na strane Urceni minima }
const MinDeltaY = 0.3;
      MinDeltaX = 2/24;
      XSteps :array [0..9] of double = (1, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01, 0.005, 0.002, 0.001);
      XPrecs :array [0..9] of byte   = (0,   1,   1,   1,    2,    2,    2,     3,     3,     3);
      VSteps :array [0..9] of double = (10, 5, 2, 1, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01);
      VPrecs :array [0..9] of byte   = ( 0, 0, 0, 0,   1,   1,   1,    2,    2,    2);
var x,x1,x2,i,y,w,h,hodmin,hodmax,valmin,valmax,vprec,xprec:integer;
    j,dt,kt,qt,kv,qv,vmin,vmax,vstep,val,dv,tmin,tmax,time1,time2,tstep:double;
    Str:String;
    m:byte;
begin
 { Zjisti hranice grafu }
 W:=MainForm.Graph.Width;
 H:=MainForm.Graph.Height;
 with MainForm,MainForm.Grid do begin
   if JDMin<2400000 then JDMin:=int(DateTimeToJD(MainForm.Datum.Date))+frac(JDMin);
   TMin:=JDMin; TMax:=JDMin;
   if not GetValue(1,VMin,m) then VMin:=0;
   if not GetValue(1,VMax,m) then VMax:=0;
   for i:=1 to RowCount-1 do if (Length(Cells[1,i])>0)and(Length(Cells[2,i])>0) then begin
     { Pro kazdy radek }
     if GetTime(i, Time1) and GetValue(i, Val, m) then begin
       Time2:=2*JDMin-Time1;
       if Time1<TMin then TMin:=Time1; if Time1>TMax then TMax:=Time1;
       if Time2<TMin then TMin:=Time2; if Time2>TMax then TMax:=Time2;
       if Val  <VMin then VMin:=Val  ; if Val  >VMax then VMax:=Val  ;
     end;
   end;
 end;
 { Uprav meze x, aby graf nebyl mensi nez 2 hodiny }
 if (TMax-TMin)<MinDeltaX then begin
   DT:=(MinDeltaX-(TMax-TMin))/2;
   TMax:=TMax+DT;
   TMin:=TMin-DT;
 end;
 i:=Length(XSteps)-1;
 while (i>0)and(((TMax-TMin)/XSteps[i])>8) do i:=i-1;
 TStep:=XSteps[i];
 XPrec:=XPrecs[i];
 TMin:=floor(TMin/TStep)*TStep;
 TMax:=ceil(TMax/TStep)*TStep;
 { Uprav meze y, aby graf nebyl mensi nez MinDeltaY stupnu }
 if ((VMax-VMin)<MinDeltaY) then begin
   DV:=(MinDeltaY-(VMax-VMin))/2;
   VMax:=VMax+DV;
   VMin:=VMin-DV;
 end;
 i:=Length(VSteps)-1;
 while (i>0)and(((VMax-VMin)/VSteps[i])>10) do i:=i-1;
 VStep:=VSteps[i];
 VPrec:=VPrecs[i];
 VMin:=floor(VMin/VStep)*VStep;
 VMax:=ceil(VMax/VStep)*VStep;
 { Spocita mapovaci konstanty }
 KT:=(W-32-32)/(TMax-TMin);
 QT:=32-KT*TMin;
 KV:=(H-16-32)/(VMax-VMin);
 QV:=24-KV*VMin;
 with MainForm.Graph.Canvas do begin
   { Vymaze kreslici plochu }
   Pen.Color:=clBlack;
   Brush.Color:=clWhite;
   FillRect(Rect(0,0,W,H));
   { Nakresli osy }
   MoveTo(32,H-24); LineTo(W-16,H-24);
   MoveTo(32,H-24); LineTo(32,16);
   { Popise osu x }
   j:=floor(TMin/TStep)*TStep;
   while (j<TMax+TStep) do begin
     X:=round(KT*j+QT);
     if (X>=32) and (X<=W-16) then begin
       MoveTo(X,H-20); LineTo(X,H-24);
       Str:=Format('%.*f',[XPrec,j-2400000]);
       TextOut(X-(TextWidth(Str) div 2),H-16,Str);
     end;
     j:=j+TStep;
   end;
   { Popise osu y }
   j:=floor(VMin/VStep)*VStep;
   while (j<VMax+VStep) do begin
     Y:=round(KV*j+QV);
     if (Y>=16) and (Y<=H-24) then begin
       MoveTo(28,Y); LineTo(32,Y);
       Y:=Y-(TextHeight('0') div 2);
       Str:=Format('%.*f',[VPrec,j]);
       TextOut(24-TextWidth(Str),Y,Str);
     end;
     j:=j+VStep;
   end;
   { Nakresli minimum }
   X:=round(KT*MainForm.JDMin+QT);
   MoveTo(X,H-24); LineTo(X,16);
   Str:='min. '+FormatDateTime('hh:nn "UT"',JDToDateTime(MainForm.JDMin));
   TextOut(X-(TextWidth(Str) div 2),4,Str);
   Brush.Style:=bsClear;
   with MainForm.Grid do
     for i:=1 to RowCount-1 do if GetTime(i, Time1) and GetValue(i, Val, M) then begin
       { Nakresli tecky }
       Time2:=2*MainForm.JDMin-Time1;         { Zrcadlovy obraz }
       X1:=round(KT*Time1+QT);
       X2:=round(KT*Time2+QT);
       Y:=round(KV*Val+QV);
       if M=0 then begin
         Pen.Color:=clBlack;
         Ellipse(X1-3,Y-3,X1+3,Y+3);       { Cerna tecka je normalni obraz }
         Pen.Color:=clRed;
         Ellipse(X2-3,Y-3,X2+3,Y+3);       { Cervena tecka je zrcadlovy obraz }
       end else
       if M=1 then begin
         Pen.Color:=clBlack;
         MoveTo(X1-2,Y-2); LineTo(X1,Y+3); LineTo(X1+3,Y-3);
         Pen.Color:=clRed;
         MoveTo(X2-2,Y-2); LineTo(X2,Y+3); LineTo(X2+3,Y-3);
       end else
       if M=2 then begin
         Pen.Color:=clBlack;
         MoveTo(X1-2,Y+2); LineTo(X1,Y-3); LineTo(X1+3,Y+3);
         Pen.Color:=clRed;
         MoveTo(X2-2,Y+2); LineTo(X2,Y-3); LineTo(X2+3,Y+3);
       end else
       if M=3 then begin
         Pen.Color:=clBlack;
         MoveTo(X1-2,Y-2); LineTo(X1+3,Y+3);
         MoveTo(X1+2,Y-2); LineTo(X1-3,Y+3);
         Pen.Color:=clRed;
         MoveTo(X2-2,Y-2); LineTo(X2+3,Y+3);
         MoveTo(X2+2,Y-2); LineTo(X2-3,Y+3);
       end;
     end;
 end;
end;

Procedure TFormularCCD.UpdateGrid;
{ Aktualizace udaju v mrizkach pri zmene tabulky odhadu }
begin
 with MainForm.Grid do begin
   if (ACol=3) then Options:=Options-[goEditing]
               else Options:=Options+[goEditing];
 end;
end;

Procedure TFormularCCD.RefreshData;
var i,x:integer;
begin
 Screen.Cursor:=crHourGlass;
 with MainForm.Grid do try
   X:=0;
   for i:=1 to RowCount-1 do if (Length(Cells[1,i])>0) then begin
     RefreshRow(i);
     X:=X+1;
   end;
   MainForm.Pocet.Text:=IntToStr(X);
 finally
   Screen.Cursor:=crDefault;
 end;
end;

Procedure TFormularCCD.RefreshRow;
var jdgeo,jdhel,faze,ep:double; Str:String;
begin
 inherited;
 with MainForm.Grid do begin
   { Upravit desetinnou carku v JD }
   if pos('.',Cells[1,ARow])>0 then begin
     Str:=Cells[1,ARow];
     while (pos('.',Str)>0) do Str[pos('.',Str)]:=',';
     Cells[1,ARow]:=Str;
   end;
   { Upravit desetinnou carku v Dmag }
   if pos('.',Cells[2,ARow])>0 then begin
     Str:=Cells[2,ARow];
     while (pos('.',Str)>0) do Str[pos('.',Str)]:=',';
     Cells[2,ARow]:=Str;
   end;
   { Vypocet faze }
   if GetTime(ARow, jdgeo) and (MainForm.GetPER>0) then begin
     jdhel:=jdgeo+HelCorr(MainForm.GetRA,MainForm.GetDEC,jdgeo);
     Faze :=(jdhel-MainForm.GetM0)/MainForm.GetPER;
     Ep   :=floor(Faze);
     Faze :=Faze-Ep;
     Cells[3,ARow]:=Format('%.3f',[faze]);
   end else
     Cells[3,ARow]:='';
 end;
end;

Procedure TFormularCCD.GotoNext;
begin
 with MainForm.Grid do
   if Col=2 then begin
     if Row=RowCount-1 then begin
       RowCount:=RowCount+1;
       Cells[0,RowCount-1]:=IntToStr(RowCount-1);
     end;
     Row:=Row+1; Col:=1;
   end
   else Col:=2;
end;

Procedure TFormularCCD.TiskZahlavi;
begin
 with P do begin
   Font.Name:='Arial CE';
   Font.Size:=20;
   Font.Style:=[fsBold];
   Align:=alignCenter;
   Writeln('PROTOKOL');
   Shift(0,300);
   Font.Size:=12;
   Writeln('PRO POZOROVN ZKRYTOVCH DVOJHVZD');
   Shift(0,500);
   Font.Size:=20;
   Writeln('CCD');
   { Ramecky pro cislo protokolu }
   Move(moveLT,0,2200);
   TableBegin;
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableLine(50);
   TableRowBegin(1000);
   TableRowEnd;
   TableLine(50);
   TableEnd;
   Move(moveLT,0,2200);
   TableBegin(alignRight);
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableLine(50);
   TableRowBegin(1000);
   TableLine(50);
   TableEnd;
   { Hlavicka }
   Shift(0,500);
   Font.Size:=10;
   Font.Style:=[fsBold];
   TableBegin(alignLeft,100,25);
   TableAddDiv(50);
   TableAddCol(3800);
   TableAddDiv(50);
   TableAddCol(5600);
   TableAddDiv(50);
   TableAddCol(2000);
   TableAddDiv(50);
   TableAddCol(5600);
   TableAddDiv(50);
   TableLine(50);
   TableRowBegin;
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Promnn hvzda');
   TableCellBegin;
   Font.Style:=[];
   Write(MainForm.Hvezda.Text);
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Datum');
   TableCellBegin;
   Font.Style:=[];
   Write(DateTimeToNightDate(MainForm.Datum.Date));
   TableLine(50);
   TableRowBegin;
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Pozorovatel');
   TableCellBegin;
   Font.Style:=[];
   Write(MainForm.Pozorovatel.Text);
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Stanovit');
   TableCellBegin;
   Font.Style:=[];
   Write(MainForm.Stanoviste.Text);
   TableLine(50);
   TableRowBegin;
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Obec bydlit');
   TableCellBegin;
   Font.Style:=[];
   Write(MainForm.Bydliste.Text);
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Pstroj');
   TableCellBegin;
   Font.Style:=[];
   with MainForm do if (pos('bez',Filtr.Text)>0)or(pos('filtr',Filtr.Text)>0) then
     Write(Pristroj.Text+', CCD '+Kamera.Text+', '+Filtr.Text)
   else
     Write(Pristroj.Text+', CCD '+Kamera.Text+', filtr '+Filtr.Text);
   TableLine(50);
   TableRowBegin;
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Pozorovac podm.');
   TableCellBegin(2);
   Font.Style:=[];
   Write(MainForm.Podminky.Text);
   TableLine(50);
   TableEnd;
   Shift(0,500);
   TabStart:=PosY;
 end;
end;

Procedure TFormularCCD.TiskZapati;
var DT:TDateTime;
    R,M,D,H,Mi,S,MS:word;
begin
 DT:=JDToDateTime(MainForm.JDMin); DecodeDate(DT,R,M,D); DecodeTime(DT,H,Mi,S,MS);
 with P do begin
   Font.Name:='Arial CE';
   Font.Size:=10;
   Font.Style:=[fsBold];
   Align:=alignCenter;
   Move(moveLB,0,-2200);
   TabStop:=PosY-GetLineHeight;
   Shift(0,500);
   TableBegin(alignLeft,100,25);
   TableAddDiv(50);
   TableAddCol(1500,alignCenter);
   TableAddDiv(50);
   TableAddCol(1500,alignCenter);
   TableAddDiv(50);
   TableAddCol(1500,alignCenter);
   TableAddDiv(50);
   TableAddCol(2500,alignCenter);
   TableAddDiv(50);
   TableAddCol(1500,alignCenter);
   TableAddDiv(50);
   TableAddCol(1500,alignCenter);
   TableAddDiv(50);
   TableLine(50);
   TableRowBegin;
   TableCellBegin(3);
   Write('Okamik minima jasnosti');
   TableCellBegin(1);
   Write('Poet odh.');
   TableLine(50);
   TableRowBegin;
   TableCellBegin;
   Write('den');
   TableCellBegin;
   Write('msc');
   TableCellBegin;
   Write('rok');
   TableCellBegin;
   Write('as [UT]');
   TableCellBegin;
   Write('celk.');
   TableCellBegin;
   Write('sest.');
   TableLine(50);
   Font.Size:=12;
   TableRowBegin(700);
   TableCellBegin;
   Shift(0,100); Write(Format('%.2d',[D]));
   TableCellBegin;
   Shift(0,100); Write(Format('%.2d',[M]));
   TableCellBegin;
   Shift(0,100); Write(Format('%.4d',[R]));
   TableCellBegin;
   Shift(0,100); Write(Format('%.2dh %.2dmin',[H,Mi]));
   TableCellBegin;
   Shift(0,100); Write(MainForm.Pocet.Text);
   TableCellBegin;
   Shift(0,100); Write(MainForm.Sestupnych.Text);
   TableLine(50);
   TableEnd;
 end;
end;

Procedure TFormularCCD.TiskZahlaviTabulky;
begin
 with P do begin
   if Assigned(P.Table) then begin
     P.TableLine(50);
     P.TableEnd;
   end;
   Font.Name:='Arial CE';
   Font.Size:=10;
   Font.Style:=[];
   Move(moveLT,0,TabStart);
   TableBegin(alignLeft,100,10);
   TableAddDiv(50);
   TableAddCol(700,alignCenter);
   TableAddDiv(50);
   TableAddCol(3000,alignCenter);
   TableAddDiv(50);
   TableAddCol(2000,alignCenter);
   TableAddDiv(50);
   TableAddCol(2000,alignCenter);
   TableAddDiv(50);
   TableAddCol(5000,alignCenter);
   TableAddDiv(50);
   TableLine(50);
   Font.Style:=[fsBold];
   TableRowBegin;
   TableCellBegin;
   Write('.');
   TableCellBegin;
   Write('JD [UT]');
   TableCellBegin;
   Align:=alignLeft; Shift(500,0);
   Font.Name:='Symbol'; Write('D');
   Font.Name:='Arial CE'; Write('mag');
   TableCellBegin;
   Write('Fze');
   TableCellBegin;
   Write('Poznmka');
   Font.Style:=[];
   TableLine(50);
 end;
end;

Procedure TFormularCCD.Print;
var i:integer;
var DT:TDateTime;
    R,M,D,H,Mi,S,MS:word;
begin
 { *** OKRAJE A NASTAVENI *** }
 P.Header.Visible:=false;
 P.Footer.Visible:=false;
 P.DocClear;
 P.TopMargin:=1500;
 P.BottomMargin:=1500;
 P.LeftMargin:=2000;
 P.RightMargin:=2000;
 P.DocBegin;
 { *** TABULKA *** }
 P.Font.Name:='Arial CE';
 P.Move(moveLT,0,0);
 TiskZahlavi(P);
 TiskZapati(P);
 TiskZahlaviTabulky(P);
 with MainForm.Grid do begin
   for i:=1 to RowCount-1 do if (Length(Cells[1,i])>0)and(Length(Cells[2,i])>0) then begin
     { Pro kazdy radek }
     if P.PosY>=Self.TabStop then begin
       P.TableLine(50);
       P.TableEnd;
       P.NewPage;
       TiskZahlavi(P);
       TiskZapati(P);
       TiskZahlaviTabulky(P);
     end;
     P.TableRowBegin;
     P.TableCellBegin;
     P.Write(IntToStr(i));
     P.TableCellBegin;
     P.Write(CheckDec(Cells[1,i]));
     P.TableCellBegin;
     P.Write(CheckDec(Cells[2,i]));
     P.TableCellBegin;
     P.Write(CheckDec(Cells[3,i]));
     P.TableCellBegin;
     P.Write(Cells[4,i]);
     P.TableRowEnd;
   end;
   P.TableLine(50);
   P.TableEnd;
 end;
 { *** GRAF *** }
 P.NewPage;
 P.Font.Name:='Arial CE';
 DT:=JDToDateTime(MainForm.JDMin); DecodeDate(DT,R,M,D); DecodeTime(DT,H,Mi,S,MS);
 with P do begin
   { Popis }
   Font.Size:=14;
   Font.Style:=[];
   Align:=alignLeft;
   Write(MainForm.Hvezda.Text);
   Align:=alignCenter;
   Write(MainForm.Pozorovatel.Text);
   Align:=alignRight;
   Write(DateTimeToNightDate(MainForm.Datum.Date));
   Writeln;
   Align:=alignCenter;
   Writeln(Format('min.: %2d. %2d. %4d %.2d:%.2d UT',[D,M,R,H,Mi]));
   Shift(0,500);
   { Graf }
   FrameBegin(P.FrameWidth,13000);
   TiskGrafu(P);
   FrameEnd;
 end;
end;

Procedure TFormularCCD.TiskGrafu;
{ Vykresleni grafu na strane Urceni minima }
const MinDeltaY = 0.3;
      MinDeltaX = 2/24;
      XSteps :array [0..9] of double = (1, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01, 0.005, 0.002, 0.001);
      XPrecs :array [0..9] of byte   = (0,   1,   1,   1,    2,    2,    2,     3,     3,     3);
      VSteps :array [0..9] of double = (10, 5, 2, 1, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01);
      VPrecs :array [0..9] of byte   = ( 0, 0, 0, 0,   1,   1,   1,    2,    2,    2);
var x,i,y,w,h,vprec,xprec,Wid:integer;
    j,dt,kt,qt,kv,qv,vmin,vmax,vstep,val,dv,tmin,tmax,time,tstep:double;
    Str:String;
    M:byte;
begin
 { Zjisti hranice grafu }
 W:=P.FrameWidth;
 H:=P.FrameHeight;
 with MainForm, MainForm.Grid do begin
   if JDMin<2400000 then JDMin:=int(DateTimeToJD(Datum.Date))+frac(JDMin);
   TMin:=JDMin; TMax:=JDMin;
   if not GetValue(1, VMin, M) then VMin:=0;
   if not GetValue(1, VMax, M) then VMax:=0;
   for i:=1 to RowCount-1 do if GetTime(i, Time) and GetValue(i, Val, M) then begin
     { Pro kazdy radek }
     if Time<TMin then TMin:=Time; if Time>TMax then TMax:=Time;
     if Val <VMin then VMin:=Val ; if Val >VMax then VMax:=Val  ;
   end;
 end;
 { Uprav meze x, aby graf nebyl mensi nez 2 hodiny }
 if (TMax-TMin)<MinDeltaX then begin
   DT:=(MinDeltaX-(TMax-TMin))/2;
   TMax:=TMax+DT;
   TMin:=TMin-DT;
 end;
 i:=Length(XSteps)-1;
 while (i>0)and(((TMax-TMin)/XSteps[i])>8) do i:=i-1;
 TStep:=XSteps[i];
 XPrec:=XPrecs[i];
 TMin:=floor(TMin/TStep)*TStep;
 TMax:=ceil(TMax/TStep)*TStep;
 { Uprav meze y, aby graf nebyl mensi nez MinDeltaY stupnu }
 if ((VMax-VMin)<MinDeltaY) then begin
   DV:=(MinDeltaY-(VMax-VMin))/2;
   VMax:=VMax+DV;
   VMin:=VMin-DV;
 end;
 i:=Length(VSteps)-1;
 while (i>0)and(((VMax-VMin)/VSteps[i])>10) do i:=i-1;
 VStep:=VSteps[i];
 VPrec:=VPrecs[i];
 VMin:=floor(VMin/VStep)*VStep;
 VMax:=ceil(VMax/VStep)*VStep;
 { Spocita mapovaci konstanty }
 KT:=(W-1300-800)/(TMax-TMin);
 QT:=1300-KT*TMin;
 KV:=(H-800-1200)/(VMax-VMin);
 QV:=800-KV*VMin;
 with P do begin
   { Nakresli osy }
   Move(moveLT,1300,0);
   P.Line(0,H-1200,35);
   Move(moveLT,1300,H-1200);
   P.Line(W-1300,0,35);
   { Popise osu x }
   Font.Size:=12;
   Font.Style:=[fsBold];
   move(moveRB,0,-GetLineHeight);
   Align:=alignRight; Write('JD [UT]');
   Align:=alignLeft;
   Font.Size:=10;
   Font.Style:=[];
   j:=floor(TMin/TStep)*TStep;
   while (j<TMax+TStep) do begin
     X:=round(KT*j+QT);
     Str:=Format('%.*f',[XPrec,j-2400000]);
     Wid:=GetTextWidth(Str);
     if (X>=1300) and (X<=W-(Wid div 2)) then begin
       move(moveLT,X,H-1300); Line(0,200,35);
       move(moveLT,X-(Wid div 2),H-950);
       Write(Str);
     end;
     j:=j+TStep;
   end;
   { Popise osu y }
   Font.Size:=12;
   Font.Style:=[fsBold];
   Align:=alignLeft;
   move(moveLT,0,0);
   Font.Name:='Symbol'; Write('D');
   Font.Name:='Arial CE'; Write('mag');
   Font.Size:=10;
   Font.Style:=[];
   j:=floor(VMin/VStep)*VStep;
   while (j<VMax+VStep) do begin
     Y:=round(KV*j+QV);
     Str:=Format('%.*f',[VPrec,j]);
     Wid:=GetTextWidth(Str);
     if (Y>=500) and (Y<=H-1200) then begin
       move(moveLT,1200,Y); Line(200,0,35);
       move(moveLT,1100-Wid,Y-(GetLineHeight div 2));
       Write(Str);
     end;
     j:=j+VStep;
   end;
   { Nakresli minimum }
   Pen.Color:=clBlack;
   Brush.Color:=clBlack;
   X:=round(KT*MainForm.JDMin+QT);
   move(moveLT,X,500); Line(0,H-1200-500,15);
   { Nakresli tecky }
   for i:=1 to MainForm.Grid.RowCount-1 do
     if GetTime(i, Time) and GetValue(i, Val, M) then begin
       X:=round(KT*Time+QT);
       Y:=round(KV*Val+QV);
       if M=0 then begin       { tecka }
         move(moveLT,X,Y);
         Ellipse(150,150,true);
       end else
       if M=1 then begin
         move(moveLT,X-75,Y-75);
         Line(75,150,25);
         move(moveLT,X,Y+75);
         Line(75,-150,25);
       end else
       if M=2 then begin
         move(moveLT,X-75,Y+75);
         Line(75,-150,25);
         move(moveLT,X,Y-75);
         Line(75,150,25);
       end else
       if M=3 then begin
         move(moveLT,X-75,Y+75);
         Line(150,-150,25);
         move(moveLT,X+75,Y+75);
         Line(-150,-150,25);
       end;
     end;
 end;
end;

end.
