unit TypClass;

(*
    Protokoly 2.1
    Copyright (C) 2002  Ing. David Motl

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

interface

Uses Forms, Controls, Classes, SysUtils, XMLStream, Math, Juldat, Graphics, Tisk;

type
    TFormular = class(TObject)
    protected
       Element   :String;
       GridIndex :integer;                         { Ukazatel v tabulce Grids }
       CompIndex :integer;                         { Ukazatel v tabulce Comps }
    public
       Constructor Create;
       Function  GetTime(ARow:integer; out Value:double):boolean; virtual; abstract;
                 { Vraci cas odhadu jako jd }
       Function  GetValue(ARow:integer; out Value:double; out Mode:byte):boolean; virtual; abstract;
                 { Vraci hvezdnou velikost odhadu }
       Procedure DataStartTag(ASender:TXMLParser; AnElement:String; S:TStrings); virtual; abstract;
       Procedure DataPCData(ASender:TXMLParser; Data:String); virtual; abstract;
       Procedure ClearData; virtual;
       Procedure ClearGrid; virtual; abstract; 
       Procedure SaveData(F:TStream); virtual; abstract;
       Procedure UpdateGrid(ACol,ARow:integer); virtual;
       Procedure UpdateComp(ACol,ARow:integer); virtual;
       Procedure UpdatePocet; virtual;
       Procedure UpdateSestup; virtual; 
       Procedure GraphPaint; virtual; abstract;
       Procedure ImportData(AFileName:String); virtual;
       Procedure RefreshData; virtual;
       Procedure RefreshRow(ARow:integer);  virtual;
       Procedure UpdateCtrls; virtual;
       Procedure ExportData(AFormat:integer; AFileName:String); virtual; abstract;
       Procedure GotoNext; virtual;
       Procedure Print(P:TPrintEngine); virtual; abstract;
    end;

const
  Verze0 = '1';

  { Projekt }
  dkProjektTag  = 'PROTOKOL';

  dkTyp         = 'TYP';
  dvTypVis      = 'VIZUALNI';
  dvTypCCD      = 'CCD';
  dkVerze       = 'VERZE';
  dkHvezda      = 'HVEZDA';
  dkRA          = 'RA';
  dkDEC         = 'DEC';
  dkM0          = 'M0';
  dkPER         = 'PER';
  dkPozorovatel = 'POZOROVATEL';
  dkJmeno       = 'JMENO';
  dkBydliste    = 'BYDLISTE';
  dkStanoviste  = 'STANOVISTE';
  dkPristroj    = 'PRISTROJ';
  dkKamera      = 'KAMERA';
  dkFiltr       = 'FILTR';
  dkOznaceni    = 'OZNACENI';
  dkZvetseni    = 'ZVETSENI';
  dkDatum       = 'DATUM';
  dkPodminky    = 'PODMINKY';
  dkExpozice    = 'EXPOZICE';
  dkFOV         = 'FOV';
  dkPOznamka    = 'POZNAMKA';
  dkData        = 'DATA';
  dkCas         = 'CAS_UT';
  dkOdhad       = 'ODHAD';
  dkJD          = 'JD';
  dkDMag        = 'DMAG';
  dkFaze        = 'FAZE';
  dkOdhSt       = 'ODH_ST';
  dkOdhadu      = 'ODHADU';
  dkCelkem      = 'CELKEM';
  dkSestupnych  = 'SESTUPNYCH';
  dkMinimum     = 'MINIMUM';
  dkMapka       = 'MAPKA';
  dkDen         = 'DEN';
  dkDen1        = 'DEN1';
  dkDen2        = 'DEN2';
  dkMEsic       = 'MES';
  dkRok         = 'ROK';
  dkHodina      = 'HOD';
  dkMinuta      = 'MIN';
  dkSrovHv      = 'SROV_HV';
  dkSrovOzn     = 'OZNACENI';
  dkHelKor      = 'HELKOR';
  dkJDHel       = 'JDHEL';

implementation

Uses Main;

Constructor TFormular.Create;
begin
 inherited;
 ClearData;
end;

Procedure TFormular.ClearData;
begin
 with MainForm do begin
   FileName :='';
   Hvezda.Text:='';
   Mapka.Text:='';
   RA.Text  :='000000.0';
   DEC.Text :='+000000';
   Min.Text :='';
   Per.Text :='';
   Sestupnych.Text:='0';
   JDMin    :=int(DateTimeToJD(Datum.Date+1))+0.5;
   GridIndex:=1;
   CompIndex:=1;
 end;
end;

Procedure TFormular.UpdateGrid;
begin
end;

Procedure TFormular.UpdateComp;
begin
end;

Procedure TFormular.ImportData;
begin
end;

Procedure TFormular.RefreshData;
begin
end;

Procedure TFormular.RefreshRow;
var j:integer;
begin
 if (ARow=MainForm.Grid.RowCount-1) then with MainForm.Grid do begin
   RowCount:=RowCount+1;
   Cells[0,RowCount-1]:=IntToStr(RowCount-1);
   for j:=1 to ColCount-1 do Cells[j,RowCount-1]:='';
 end;
end;

Procedure TFormular.GotoNext;
begin
end;

Procedure TFormular.UpdatePocet;
var i,Odhadu:integer;
begin
 Odhadu:=0;
 { Zjisti pouzite srovnavaci hvezdy a jejich jasnost }
 with MainForm.Grid do begin
   { V CL se sestavi tabulka pouzitych dvojic srovnavacich hvezd, soucet rozdilu a pocet pouziti }
   for i:=1 to RowCount-1 do
     if (Length(Cells[1,i])>0) and (Length(Cells[2,i])>0) then begin
       Odhadu:=Odhadu+1;
   end;
   { Jeste vyplnit pocet pozorovani }
   MainForm.Pocet.Text:=IntToStr(Odhadu);
 end;
end;

Procedure TFormular.UpdateSestup;
var i,Sestupnych:integer; jd,val:double; M:byte;
begin
 Sestupnych:=0;
 { Zjisti pouzite srovnavaci hvezdy a jejich jasnost }
 with MainForm.Grid do begin
   { V CL se sestavi tabulka pouzitych dvojic srovnavacich hvezd, soucet rozdilu a pocet pouziti }
   for i:=1 to RowCount-1 do
     if GetTime(i, jd) and GetValue(i, val, M) then begin
       Sestupnych:=Sestupnych+ord(jd<MainForm.JDMin); { Pocitadlo sestupnych odhadu }
   end;
   { Jeste vyplnit pocet pozorovani }
   MainForm.Sestupnych.Text:=IntToStr(Sestupnych);
 end;
end;

Procedure TFormular.UpdateCtrls;
begin
end;

end.
