unit TypVis;

(*
    Protokoly 2.4
    Copyright (C) 2002  Ing. David Motl

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

interface

Uses Controls, Forms, Classes, SysUtils, XMLStream, TypClass, Math,
     Juldat, Grids, Graphics, Utils, Dialogs, Tisk;

type
  TFormularVis = class(TFormular)
  protected
     TabStart,TabStop:integer;
     Procedure CompClear;
     Procedure CompDef(AName:String; AValue:double);
               { Pridej novou srov. hvezdu }
     Function  CompValue(C:char):double;
               { Vraci hodnotu srov. hvezdy }
     Procedure ParseData(A:String; out m,n:char; out x,y:integer; out iac:boolean);
               { Rozkodovani odhadu }
     Procedure GridTime(ARow:integer; var H,M:word);
               { Doplneni neuplneho zapisu casu }
     Procedure TiskZahlavi(P:TPrintEngine);
     Procedure TiskZapati(P:TPrintEngine);
     Procedure TiskZahlaviTabulky(P:TPrintEngine);
     Procedure TiskTabulkySrovHv(P:TPrintEngine);
     Procedure TiskGrafu(P:TPrintEngine);
  public
     Constructor Create;
     Function  GetTime(ARow:integer; out Value:double):boolean; override;
               { Vraci cas odhadu jako jd }
     Function  GetValue(ARow:integer; out Value:double; out Mode:byte):boolean; override;
               { Vraci hvezdnou velikost odhadu }
     Procedure DataStartTag(ASender:TXMLParser; AnElement:String; S:TStrings); override;
     Procedure DataPCData(ASender:TXMLParser; Data:String); override;
     Procedure ClearData; override;
     Procedure ClearGrid; override;
     Procedure SaveData(F:TStream); override;
     Procedure UpdateGrid(ACol,ARow:integer); override;
     Procedure UpdateComp(ACol,ARow:integer); override;
     Procedure RefreshData; override;
     Procedure RefreshRow(ARow:integer); override;
     Procedure ExportData(AFormat:integer; AFileName:String); override;
     Procedure GraphPaint; override;
     Procedure GotoNext; override;
     Procedure Print(P:TPrintEngine); override;
  end;

implementation

Uses Main;

Constructor TFormularVis.Create;
begin
 inherited;
end;

Procedure TFormularVis.DataStartTag;
{ Zpracovani tagu pri cteni proejktu }
var d,r:integer; Yr,Mo,Da,h,m:word; DT:TDateTime;
begin
 Element:=AnElement;
 if Element=dkData then with MainForm.Grid do begin
   { Odhady }
   RowCount:=max(RowCount,GridIndex+1);
   Cells[0,GridIndex]:=IntToStr(GridIndex);
   Cells[1,GridIndex]:=XMLString(S,dkCas,'');
   Cells[2,GridIndex]:=XMLString(S,dkOdhad,'');
   Cells[3,GridIndex]:=XMLString(S,dkOdhSt,'');
   Cells[4,GridIndex]:=XMLString(S,dkPoznamka,'');
   GridIndex:=GridIndex+1;
 end else
 if Element=dkPristroj then with MainForm do begin
   { Pristroj }
   Pristroj.Text:=XMLString(S,dkOznaceni,'');
   Zvetseni.Text:=XMLString(S,dkZvetseni,'');
 end else
 if Element=dkPozorovatel then with MainForm do begin
   { Pozorovatel }
   Pozorovatel.Text:=XMLString(S,dkJmeno,'');
   Bydliste.Text:=XMLString(S,dkBydliste,'');
 end else
 if Element=dkDatum then with MainForm do begin
   { Datum pozorovani }
   D:=XMLInteger(S,dkDen2,1);
   M:=XMLInteger(S,dkMesic,1);
   R:=XMLInteger(S,dkRok,2000);
   Datum.Date:=encodeDate(R,M,D)-1;
 end else
 if Element=dkMinimum then with MainForm do begin
   { Okamzik minima - muze byt uveden JD nebo pouze jen JD }
   try
   if S.IndexOfName(dkJD)>=0 then begin
       JDMin:=StrToFloatDef(CheckDec(XMLString(S,dkJD,'')),0);
       if (JDMin<30000) then begin
         MessageDlg('V souboru je uloen chybn okamik minima.',mtWarning,[mbOK],0);
         JDMin:=int(DateTimeToJD(Datum.Date))+0.5;
       end else
       if JDMin<2000000 then begin
         JDMin:=JDMin+2400000;
       end;
   end else begin
     H:=XMLInteger(S,dkHodina,0);
     M:=XMLInteger(S,dkMinuta,0);
     DecodeDate(Datum.Date,Yr,Mo,Da);
     Yr:=XMLInteger(S,dkRok,Yr);
     Mo:=XMLInteger(S,dkMesic,Mo);
     Da:=XMLInteger(S,dkDen,Da);
     if H>12 then DT:=EncodeDate(Yr,Mo,Da)+EncodeTime(H,M,0,0)
             else DT:=EncodeDate(Yr,Mo,Da)+EncodeTime(H,M,0,0)+1;
     JDMin:=DateTimeToJD(DT);
     if JDMin<2000000 then JDMin:=JDMin+2400000;
   end;
   except
     JDMin:=2452000.5;
   end;
 end;
end;

Procedure TFormularVis.DataPCData;
{ Nacti data, podle toho, jaky byl naposledy element }
begin
 with MainForm do begin
   if Element=dkHvezda     then Hvezda.Text:=Data else
   if Element=dkStanoviste then Stanoviste.Text:=Data else
   if Element=dkPodminky   then Podminky.Text:=Data else
   if Element=dkMapka      then Mapka.Text:=Data else
   if Element=dkOdhadu     then Pocet.Text:=Data else
   if Element=dkSestupnych then Sestupnych.Text:=Data;
 end;
end;

Procedure TFormularVis.ClearData;
var i,j:integer;
begin
 inherited;
 with MainForm, Grid do begin
   CompSheet.TabVisible:=true;
   Kamera.Visible:=false;
   Filtr.Visible :=false;
   Zvetseni.Visible:=true;
   ToolButton8.Visible:=false;
   ToolButton12.Visible:=false;
   Mapka.Visible:=true;
   Katalog.Visible:=false;
   { Vymazani tabulky Grid }
   ClearGrid;
 end;
 with MainForm.Comps do begin
   { Vymazani tabulky Comps }
   DefaultRowHeight:=Canvas.TextHeight('')+4;
   ColCount:=3;
   RowCount:=6;
   { Nastaveni sloupcu }
   Cells[0,0]:='Oznaen';     ColWidths[0]:=70;
   Cells[1,0]:='Odh. st.';     ColWidths[1]:=70;
   Cells[2,0]:='Poznmka';     ColWidths[2]:=Width-170;
   { Vymazani dat }
   for i:=0 to 2 do for j:=1 to 5 do Cells[i,j]:='';
 end;
 UpdateGrid(MainForm.Grid.Col,MainForm.Grid.Row);
 UpdateComp(MainForm.Comps.Col,MainForm.Comps.Row)
end;

Procedure TFormularVis.SaveData;
{ Ulozeni projektu ve formatu XML }
var i:integer; r,m,d,h,mi,s,ms:word; jd,dt:double;
begin
 XMLHeader(F);
 XMLBeginTag(F,dkProjektTag);
 with MainForm do begin
   { Hlavicka }
   XMLAttribute(F,dkTyp,dvTypVis);
   XMLAttribute(F,dkVerze,Verze0);
   XMLDoneTag(F,false);
   { Hvezda }
   XMLSimpleData(F,dkHvezda,Hvezda.Text);
   { Pozorovatel }
   XMLBeginTag(F,dkPozorovatel);
     XMLAttribute(F,dkJmeno,Pozorovatel.Text);
     XMLAttribute(F,dkBydliste,Bydliste.Text);
     XMLDoneTag(F,true);
   { Stanoviste }
   XMLSimpleData(F,dkStanoviste,Stanoviste.Text);
   { Pristroj }
   XMLBeginTag  (F,dkPristroj);
     XMLAttribute(F,dkOznaceni,Pristroj.Text);
     XMLAttribute(F,dkZvetseni,Zvetseni.Text);
     XMLDoneTag(F,true);
   { Datum pozorovani }
   XMLBeginTag  (F,dkDatum);
     DecodeDate(Datum.Date+1,R,M,D);
     JD:=DateTimeToJD(Datum.Date+1);
     XMLAttribute(F,dkDen1,D-1);
     XMLAttribute(F,dkDen2,D);
     XMLAttribute(F,dkMesic,M);
     XMLAttribute(F,dkRok,R);
     XMLAttribute(F,dkJD,SetDec(Format('%.1f',[JD])));
     XMLDoneTag(F,true);
   { Podminky }
   XMLSimpleData(F,dkPodminky,Podminky.Text);
   { Data = odhady / mereni }
   with Grid do begin
     for i:=1 to RowCount-1 do if (Length(Cells[1,i])>0) and (Length(Cells[2,i])>0) then begin
       XMLBeginTag(F,dkData);
         XMLAttribute(F,dkCas  ,Cells[1,i]);
         XMLAttribute(F,dkOdhad,Cells[2,i]);
         XMLAttribute(F,dkOdhSt,SetDec(Cells[3,i]));
         if Length(Cells[4,i])>0 then XMLAttribute(F,dkPoznamka,Cells[4,i]);
       XMLDoneTag(F,true);
     end;
   end;
   { Srovnavaci hvezdy }
   with Comps do begin
     for i:=1 to RowCount-1 do if (Length(Cells[0,i])>0) and (Length(Cells[1,i])>0) then begin
       XMLBeginTag(F,dkSrovHv);
         XMLAttribute(F,dkSrovOzn,Cells[0,i]);
         XMLAttribute(F,dkOdhSt,SetDec(Cells[1,i]));
         if Length(Cells[2,i])>0 then XMLAttribute(F,dkPoznamka,Cells[2,i]);
       XMLDoneTag(F,true);
     end;
   end;
   { Mapka }
   XMLSimpleData(F,dkMapka,Mapka.Text);
   { Okamzik minima }
   XMLBeginTag(F,dkMinimum);
     XMLAttribute(F,dkJD,SetDec(Format('%.4f',[JDMin])));
     try
       DT:=JDToDateTime(JDMin);
       DecodeDate(DT,R,M,D); DecodeTime(DT,H,Mi,S,MS);
       XMLAttribute(F,dkRok,R);
       XMLAttribute(F,dkMesic,M);
       XMLAttribute(F,dkDen,D);
       XMLAttribute(F,dkHodina,H);
       XMLAttribute(F,dkMinuta,Mi);
     except
     end;
     XMLDoneTag(F,true);
   { Pocet odhadu }
   XMLSimpleData(F,dkOdhadu,Pocet.Text);
   XMLSimpleData(F,dkSestupnych,Sestupnych.Text);
 end;
 XMLEndTag(F,dkProjektTag);
end;

Procedure TFormularVis.UpdateGrid;
{ Aktualizace udaju v mrizkach pri zmene tabulky odhadu }
begin
 with MainForm.Grid do begin
   if (ACol=3) then Options:=Options-[goEditing]
               else Options:=Options+[goEditing];
 end;
end;

Procedure TFormularVis.GridTime;
{ Prevod casu pri neuplnem zadani v mrizce Grid.
  Povolene formaty: "mm", "hmm", " mm", ":mm", "hhmm", "hh:mm" "hh mm" }
var i,x,y,z:integer; Value:String; LMi:word;
begin
 H:=0; M:=0;
 if (ARow<=0) then exit;
 Value:=MainForm.Grid.Cells[1,ARow];
 x:=pos(':',Value); y:=pos(' ',Value); z:=Length(Value);
 if ((x=0)and(y=0)and(z<=2))or((x=1)or(y=1)) then begin
   { Nedal zadny oddelovac, delka do dvou cislic }
   M:=StrToIntDef(Trim(copy(Value,1+ord((x>0)or(y>0)),2)),0);
   { hledej v predchazejicich zaznamech, kde je uvedena hodina }
   for i:=ARow-1 downto 1 do begin
     { Tady je specifikovan uplny cas }
     GridTime(i,H,LMi);
     { Kdyz jsou mensi minuty, posun o hodinu }
     if LMi>=M then H:=H+1;
     { Kdyz se prekroci pulnoc, nastavi se hodiny na 0 }
     if H>=24 then H:=0;
     break;
   end;
 end else
 if (x=0)and(y=0)and(z>2) then begin
   { Nedal oddelovac, ale dal dve nebo tri cislice (hmm, hhmm) }
   H:=StrToIntDef(Trim(copy(Value,1,z-2)),0);
   M:=StrToIntDef(Trim(copy(Value,z+1,2)),0);
 end else
 if (y>1) then begin
   { Dal oddelovac mezeru }
   H:=StrToIntDef(Trim(copy(Value,1,y-1)),0);
   M:=StrToIntDef(Trim(copy(Value,y+1,2)),0);
 end else
 if (x>1) then begin
   { Dal oddelovac dvojtecku }
   H:=StrToIntDef(Trim(copy(Value,1,x-1)),0);
   M:=StrToIntDef(Trim(copy(Value,x+1,2)),0);
 end;
 { Uprava do spravnych mezi }
 H:=Max(0,Min(23,H));
 M:=Max(0,Min(59,M));
end;

Procedure TFormularVis.ParseData;
{ Dekodovani zapisu odhadu. Povolene formaty:
    a22v33b
    a22v
    v33b
    <d
    >a
}
var len, i, v, lt, gt:integer; Aux:String;
begin
 Len:=Length(A);
 iac:=pos(':',A)>0;
 for i:=1 to Len do begin
   if A[i] in ['A'..'Z','a'..'z','0'..'9','<','>'] then Aux:=Aux+A[i];
 end;
 Len:=Length(Aux);
 gt:=pos('>', Aux);
 lt:=pos('<', Aux);
 v:=pos('v',Aux);
 if lt=1 then begin  { <n }
   M:=#0;
   N:=Aux[Len];
   X:=0;
   Y:=-1;
 end else
 if gt=1 then begin  { >m }
   M:=Aux[Len];
   N:=#0;
   X:=-1;
   Y:=0;
 end else
 if v=1 then begin { vy..yn }
   M:=#0;
   N:=Aux[Len];
   X:=0;
   Y:=StrToIntDef(copy(Aux,2,Len-2),0);
 end else
 if v=Length(Aux) then begin { mx..xv }
   M:=Aux[1];
   N:=#0;
   X:=StrToIntDef(copy(Aux,2,Len-2),0);
   Y:=0;
 end else begin   { mx..xvy..yn }
   M:=Aux[1];
   N:=Aux[Len];
   X:=StrToIntDef(copy(Aux,2,v-2),0);
   Y:=StrToIntDef(copy(Aux,v+1,Len-v-1),0);
 end;
end;

Function TFormularVis.CompValue;
{ Precte z Comps hodnotu srovnavaci hvezdy }
var i:integer;
begin
 Result:=0;
 with MainForm.Comps do for i:=1 to RowCount-1 do
   if Cells[0,i]=C then begin Result:=StrToFloatDef(CheckDec(Cells[1,i]),0); break; end;
end;

Procedure TFormularVis.CompClear;
{ Vymaze tabulku srovnavacich hvezd }
var i:integer;
begin
 CompIndex:=1;
 with MainForm.Comps do for i:=1 to RowCount-1 do begin
   Cells[0,i]:='';
   Cells[1,i]:='';
 end;
end;

Procedure TFormularVis.CompDef;
{ Prida do tabulky srovnavaci hvezdu }
begin
 { Realokuj misto, pokud je to potreba }
 if CompIndex>=MainForm.Comps.RowCount then
   MainForm.Comps.RowCount:=CompIndex+1;
 with MainForm.Comps do begin
   Cells[0,CompIndex]:=AName;
   Cells[1,CompIndex]:=Format('%.1f',[AValue]);
   CompIndex:=CompIndex+1;
 end;
end;

procedure TFormularVis.UpdateComp;
{ V tabulce Comps jdou editovat jen poznamky (sloupec 2) }
begin
 with MainForm.Comps do begin
   if Col=2 then Options:=Options+[goEditing] else
                 Options:=Options-[goEditing];
 end;
end;

Procedure TFormularVis.RefreshData;
var x,y,i,Index,Sum,Cnt:integer; N,M,Brightest:char;
    CL:TStringList; Val,W,V:double; Done,L,R:String; ok,inac:boolean; H,Mi:word;
begin
 { Zjisti pouzite srovnavaci hvezdy a jejich jasnost }
 CompClear;
 CL:=TStringList.Create;
 Screen.Cursor:=crHourGlass;
 with MainForm.Grid do try
   { V CL se sestavi tabulka pouzitych dvojic srovnavacich hvezd, soucet rozdilu a pocet pouziti }
   for i:=1 to RowCount-1 do begin
     if (Length(Cells[1,i])>0) then begin
       GridTime(i,H,Mi);
       MainForm.Grid.Cells[1,i]:=Format('%.2d:%.2d',[H,Mi]);
     end;
     if (Length(Cells[1,i])>0) and (Length(Cells[2,i])>0) then begin
       ParseData(Cells[2,i],M,N,X,Y,inac);
       if (M>' ')and(N>' ') then begin { Musi mit obe strany definovane }
         Index:=CL.IndexOf(M+N);
         if Index>=0 then begin
           { Uz tam je }
           Sum:=integer(CL.Objects[Index]) mod 65536;
           Cnt:=integer(CL.Objects[Index]) div 65536;
           Sum:=Sum+X+Y;
           Cnt:=Cnt+1;
           CL.Objects[Index]:=pointer(Sum+Cnt*65536);
         end else begin
           { Jeste nebyla pouzita }
           CL.AddObject(M+N,pointer(X+Y));
           L:=L+M;
           R:=R+N;
         end;
       end;
     end;
   end;
   if CL.Count>0 then begin
     { Najde nejjasnejsi hvezdu - je na leve strane a neni na prave }
     Brightest:=#0;
     for i:=1 to Length(L) do begin
       if pos(L[i],R)=0 then begin Brightest:=L[i]; break; end;
     end;
     if Brightest>' ' then begin
       { Ulozi nejjasnejsi srovnavaci hvezdu }
       Val:=0;        { Srovnavaci a ma odh. st. = 0.0 }
       CompDef(Brightest,Val);
       Done:=Brightest;       { Jiz zpracovane srovnavaci hvezdy }
       repeat
         { Najde zaznam s nejjasnejsi hvezdou vlevo }
         ok:=false;
         for i:=0 to CL.Count-1 do if (CL[i][1]=Brightest)and(pos(CL[i][2],Done)=0) then begin
           Brightest:=CL[i][2]; { Bude se pokracovat hvezdou vpravo }
           Sum:=integer(CL.Objects[I]) mod 65536;
           Cnt:=integer(CL.Objects[I]) div 65536;
           { Sum je soucet hodnot x+y, Cnt je pocet odhadu -1 }
           Val      :=Val+Sum/(Cnt+1);
           { Pridej dalsi srovnavaci hvezdu }
           CompDef(Brightest,Val);
           { Pri dalsi hledani se ignoruje hvezdy v Done aby nevznikla nekonecna smycka }
           Done:=Done+Brightest;
           ok:=true;
           break;
         end;
       until not ok;
     end;
   end;
   { Priradi hodnoty k radkum s pozorovanim }
   for i:=1 to RowCount-1 do begin
     if (Length(Cells[1,i])>0) and (Length(Cells[2,i])>0) then begin
       { Dekoduje odhad }
       ParseData(Cells[2,i],M,N,X,Y,inac);
       Cells[3,i]:='';
       if (M>' ')and(N>' ')and(X+Y>0) then begin
         { Ma obe strany definovane }
         V:=CompValue(M); W:=CompValue(N);
         Cells[3,i]:=Format('%.1f',[V+(W-V)/(X+Y)*X]);
       end else
       if (M>' ')and(X<0) then begin
         { Jasnejsi nez srovnavacka }
         V:=CompValue(M);
         Cells[3,i]:=Format('>%.1f',[V]);
       end else
       if (N>' ')and(Y<0) then begin
         { Slabsi nez srovnavacka }
         W:=CompValue(N);
         Cells[3,i]:=Format('<%.1f',[W]);
       end else
       if (M>' ') then begin
         { Ma levou stranu definovanou }
         V:=CompValue(M);
         Cells[3,i]:=Format('%.1f',[V+X]);
       end else
       if (N>' ') then begin
         { Ma pravou stranu definovanou }
         W:=CompValue(N);
         Cells[3,i]:=Format('%.1f',[W-Y]);
       end;
       if inac then Cells[3,i]:=Cells[3,i]+':';
     end else
       { Neni uveden cas nebo odhad }
       Cells[3,i]:='';
   end;
   UpdatePocet;
 finally
   CL.Free;
   Screen.Cursor:=crDefault;
 end;
end;

Function TFormularVis.GetTime;
var x,y,H,mi:integer; Val:String; DT:TDateTime;
begin
 Val:=MainForm.Grid.Cells[1,ARow];
 x:=pos(':',Val); y:=pos(' ',Val);
 H:=0; Mi:=0;
 if (x<=1)and(y<=1) then begin
   { nedal oddelovac nebo je oddelovac prvni znak }
   if (x=1)or(y=1) then Mi:=StrToIntDef(Trim(copy(Val,2,2)),0)
                   else Mi:=StrToIntDef(Trim(copy(Val,1,2)),0);
 end else
 if (y>1) then begin { Dal oddelovac mezeru }
   H:=StrToIntDef(Trim(copy(Val,1,y-1)),0);
   Mi:=StrToIntDef(Trim(copy(Val,y+1,2)),0);
 end else
 if (x>1) then begin { Dal oddelovac dvojtecku }
   H:=StrToIntDef(Trim(copy(Val,1,x-1)),0);
   Mi:=StrToIntDef(Trim(copy(Val,x+1,2)),0);
 end;
 { Uprava do spravnych mezi }
 H:=Max(0,Min(23,H));
 Mi:=Max(0,Min(59,Mi));
 { Vypocet DateTime }
 if H>=12 then
   DT:=int(MainForm.Datum.Date)+(H/24)+(Mi/24/60)
 else
   DT:=int(MainForm.Datum.Date+1)+(H/24)+(Mi/24/60);
 { Prevod na juldat }
 Value:=DateTimeToJD(DT);
 Result:=(Value>0);
end;

Function TFormularVis.GetValue;
var Aux:String;
begin
 Aux:=CheckDec(MainForm.Grid.Cells[3,ARow]);
 Mode:=0;
 if copy(Aux,1,1)='<' then begin
   Delete(Aux,1,1);
   Mode:=1;
 end else
 if copy(Aux,1,1)='>' then begin
   Delete(Aux,1,1);
   Mode:=2;
 end else
 if (Length(Aux)>0) and (copy(Aux,Length(Aux),1)=':') then begin
   Delete(Aux,Length(Aux),1);
   Mode:=3;
 end;
 Result:=TryStrToFloat(Aux,Value);
end;

Procedure TFormularVis.ExportData;
var F:TextFile; R:integer; jd,v:double; line:String; m:byte;
begin
 AssignFile(F,AFileName); rewrite(f);
 try
   with MainForm.Grid do for R:=1 to RowCount-1 do begin
     if GetTime(R, jd) and GetValue(R, v, m) then begin
       if M<>0 then v:=99.999;
       if AFormat=1 then jd:=frac(jd/10)*10;
       line:=Format('%.5f %.3f',[jd,v]);
       if AFormat<>2 then
         while pos(',',line)>0 do line[pos(',',line)]:='.';
       Writeln(f,line);
     end;
   end;
 finally
   closeFile(f);
 end;
end;

procedure TFormularVis.GraphPaint;
{ Vykresleni grafu na strane Urceni minima }
const MinDeltaX = 3/24;
      MinDeltaY = 8;
      XSteps :array [0..11] of double = (1, 12/24, 6/24, 3/24, 2/24, 1/24, 30/1440, 15/1440, 10/1440, 5/1440, 2/1440, 1/1440);
      VSteps :array [0..9] of double = (10, 5, 2, 1, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01);
      VPrecs :array [0..9] of byte = (0, 0, 0, 0, 1, 1, 1, 2, 2, 2);
var x,x1,x2,i,y,w,h,hodmin,hodmax,valmin,valmax,vprec:integer;
    j,dt,kt,qt,kv,qv,vmin,vmax,vstep,val,dv,tmin,tmax,time1,time2,tstep:double;
    Str:String;
    M:byte;
begin
 { Zjisti hranice grafu }
 W:=MainForm.Graph.Width;
 H:=MainForm.Graph.Height;
 with MainForm,MainForm.Grid do begin
   TMin:=JDMin; TMax:=JDMin;
   if not GetValue(1, VMin, M) then VMin:=0;
   if not GetValue(1, VMax, M) then VMax:=0;
   for i:=1 to RowCount-1 do if GetTime(i, Time1) and GetValue(i, Val, M) then begin
     { Pro kazdy radek }
     Time2:=2*JDMin-Time1;
     if Time1<TMin then TMin:=Time1; if Time1>TMax then TMax:=Time1;
     if Time2<TMin then TMin:=Time2; if Time2>TMax then TMax:=Time2;
     if Val  <VMin then VMin:=Val  ; if Val  >VMax then VMax:=Val  ;
   end;
 end;
 { Uprav meze x, aby graf nebyl mensi nez 2 hodiny }
 if (TMax-TMin)<MinDeltaX then begin
   DT:=(MinDeltaX-(TMax-TMin))/2;
   TMax:=TMax+DT;
   TMin:=TMin-DT;
 end;
 i:=Length(XSteps)-1;
 while (i>0)and(((TMax-TMin)/XSteps[i])>10) do i:=i-1;
 TStep:=XSteps[i];
 TMin:=floor(TMin/TStep)*TStep;
 TMax:=ceil(TMax/TStep)*TStep;
 { Uprav meze y, aby graf nebyl mensi nez MinDeltaY stupnu }
 if ((VMax-VMin)<MinDeltaY) then begin
   DV:=(MinDeltaY-(VMax-VMin))/2;
   VMax:=VMax+DV;
   VMin:=VMin-DV;
 end;
 i:=Length(VSteps)-1;
 while (i>0)and(((VMax-VMin)/VSteps[i])>10) do i:=i-1;
 VStep:=VSteps[i];
 VPrec:=VPrecs[i]; 
 VMin:=floor(VMin/VStep)*VStep;
 VMax:=ceil(VMax/VStep)*VStep;
 { Spocita mapovaci konstanty }
 KT:=(W-32-24)/(TMax-TMin);
 QT:=32-KT*TMin;
 KV:=(H-16-32)/(VMax-VMin);
 QV:=24-KV*VMin;
 with MainForm.Graph.Canvas do begin
   { Vymaze kreslici plochu }
   Pen.Color:=clBlack;
   Brush.Color:=clWhite;
   Brush.Style:=bsSolid;
   FillRect(Rect(0,0,W,H));
   { Nakresli osy }
   MoveTo(32,H-24); LineTo(32,16);
   MoveTo(32,H-24); LineTo(W-16,H-24);
   { Popise osu x }
   j:=floor(TMin/TStep)*TStep;
   while (j<TMax+TStep) do begin
     X:=round(KT*j+QT);
     if (X>=32) and (X<=W-16) then begin
       MoveTo(X,H-20); LineTo(X,H-24);
       Str:=FormatDateTime('hh:nn',JDToDateTime(j));
       TextOut(X-(TextWidth(Str) div 2),H-16,Str);
     end;
     j:=j+TStep;
   end;
   { Popise osu y }
   j:=floor(VMin/VStep)*VStep;
   while (j<VMax+VStep) do begin
     Y:=round(KV*j+QV);
     if (Y>=16) and (Y<=H-24) then begin
       MoveTo(28,Y); LineTo(32,Y);
       Y:=Y-(TextHeight('0') div 2);
       Str:=Format('%.*f',[VPrec,j]);
       TextOut(24-TextWidth(Str),Y,Str);
     end;
     j:=j+VStep;
   end;
   { Nakresli minimum }
   X:=round(KT*MainForm.JDMin+QT);
   MoveTo(X,H-16); LineTo(X,16);
   Str:='min. '+FormatDateTime('hh:nn "UT"',JDToDateTime(MainForm.JDMin));
   TextOut(X-(TextWidth(Str) div 2),4,Str);
   Brush.Style:=bsClear;
   with MainForm.Grid do
     for i:=1 to RowCount-1 do if GetTime(i, Time1) and GetValue(i, Val, M) then begin
       { Nakresli tecky }
       Time2:=2*MainForm.JDMin-Time1;  { Zrcadlovy obraz }
       X1:=round(KT*Time1+QT);
       X2:=round(KT*Time2+QT);
       Y:=round(KV*Val+QV);
       if M=0 then begin
         Pen.Color:=clBlack;
         Ellipse(X1-3,Y-3,X1+3,Y+3);       { Cerna tecka je normalni obraz }
         Pen.Color:=clRed;
         Ellipse(X2-3,Y-3,X2+3,Y+3);       { Cervena tecka je zrcadlovy obraz }
       end else
       if M=1 then begin
         Pen.Color:=clBlack;
         MoveTo(X1-2,Y-2); LineTo(X1,Y+3); LineTo(X1+3,Y-3);
         Pen.Color:=clRed;
         MoveTo(X2-2,Y-2); LineTo(X2,Y+3); LineTo(X2+3,Y-3);
       end else
       if M=2 then begin
         Pen.Color:=clBlack;
         MoveTo(X1-2,Y+2); LineTo(X1,Y-3); LineTo(X1+3,Y+3);
         Pen.Color:=clRed;
         MoveTo(X2-2,Y+2); LineTo(X2,Y-3); LineTo(X2+3,Y+3);
       end else
       if M=3 then begin
         Pen.Color:=clBlack;
         MoveTo(X1-2,Y-2); LineTo(X1+3,Y+3);
         MoveTo(X1+2,Y-2); LineTo(X1-3,Y+3);
         Pen.Color:=clRed;
         MoveTo(X2-2,Y-2); LineTo(X2+3,Y+3);
         MoveTo(X2+2,Y-2); LineTo(X2-3,Y+3);
       end;
     end;
 end;
end;

Procedure TFormularVis.RefreshRow;
begin
 inherited;
 RefreshData;
end;

Procedure TFormularVis.GotoNext;
begin
 with MainForm.Grid do begin
   if Col=2 then begin
     Row:=Row+1;
     Col:=1;
   end else
     Col:=2;
 end;     
end;

Procedure TFormularVis.ClearGrid;
{ Vymazani dat z tabulek }
var i,j:integer;
begin
 with MainForm, Grid do begin
   DefaultRowHeight:=Canvas.TextHeight('')+4;
   ColCount:=5;
   RowCount:=2;
   { Nastaveni sloupcu }
   Cells[0,0]:='slo';     ColWidths[0]:=40;
   Cells[1,0]:='as [UT]';  ColWidths[1]:=100;
   Cells[2,0]:='Odhad';     ColWidths[2]:=120;
   Cells[3,0]:='Odh. st.';  ColWidths[3]:=100;
   Cells[4,0]:='Poznmka';  ColWidths[4]:=Width-390;
   { Nastaveni radku }
   for i:=1 to RowCount-1 do Cells[0,i]:=IntToStr(i);
   { Vymazani dat }
   for i:=1 to ColCount-1 do for j:=1 to RowCount-1 do Cells[i,j]:='';
 end;
 with MainForm.Comps do begin
   { Vymazani tabulky Comps }
   DefaultRowHeight:=Canvas.TextHeight('')+4;
   ColCount:=3;
   RowCount:=6;
   { Nastaveni sloupcu }
   Cells[0,0]:='Oznaen';     ColWidths[0]:=70;
   Cells[1,0]:='Odh. st.';     ColWidths[1]:=70;
   Cells[2,0]:='Poznmka';     ColWidths[2]:=Width-170;
   { Vymazani dat }
   for i:=0 to 2 do for j:=1 to 5 do Cells[i,j]:='';
 end;
 UpdateGrid(MainForm.Grid.Col,MainForm.Grid.Row);
 UpdateComp(MainForm.Comps.Col,MainForm.Comps.Row);
end;

Procedure TFormularVis.TiskZahlavi;
begin
 with P do begin
   Font.Name:='Arial CE';
   Font.Size:=20;
   Font.Style:=[fsBold];
   Align:=alignCenter;
   Writeln('PROTOKOL');
   Shift(0,300);
   Font.Size:=12;
   Writeln('PRO POZOROVN ZKRYTOVCH DVOJHVZD');
   { Ramecky pro cislo protokolu }
   Move(moveLT,0,2200);
   TableBegin;
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableLine(50);
   TableRowBegin(1000);
   TableRowEnd;
   TableLine(50);
   TableEnd;
   Move(moveLT,0,2200);
   TableBegin(alignRight);
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableAddCol(1500);
   TableAddDiv(50);
   TableLine(50);
   TableRowBegin(1000);
   TableLine(50);
   TableEnd;
   { Hlavicka }
   Shift(0,500);
   Font.Size:=10;
   Font.Style:=[fsBold];
   TableBegin(alignLeft,100,25);
   TableAddDiv(50);
   TableAddCol(3800);
   TableAddDiv(50);
   TableAddCol(5600);
   TableAddDiv(50);
   TableAddCol(2000);
   TableAddDiv(50);
   TableAddCol(5600);
   TableAddDiv(50);
   TableLine(50);
   TableRowBegin;
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Promnn hvzda');
   TableCellBegin;
   Font.Style:=[];
   Write(MainForm.Hvezda.Text);
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Datum');
   TableCellBegin;
   Font.Style:=[];
   Write(DateTimeToNightDate(MainForm.Datum.Date));
   TableLine(50);
   TableRowBegin;
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Pozorovatel');
   TableCellBegin;
   Font.Style:=[];
   Write(MainForm.Pozorovatel.Text);
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Stanovit');
   TableCellBegin;
   Font.Style:=[];
   Write(MainForm.Stanoviste.Text);
   TableLine(50);
   TableRowBegin;
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Obec bydlit');
   TableCellBegin;
   Font.Style:=[];
   Write(MainForm.Bydliste.Text);
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Pstroj');
   TableCellBegin;
   Font.Style:=[];
   Write(MainForm.Pristroj.Text+', zv. '+MainForm.Zvetseni.Text+'x');
   TableLine(50);
   TableRowBegin;
   TableCellBegin;
   Font.Style:=[fsBold];
   Write('Pozorovac podm.');
   TableCellBegin(2);
   Font.Style:=[];
   Write(MainForm.Podminky.Text);
   TableLine(50);
   TableEnd;
   Shift(0,500);
   TabStart:=PosY;
 end;
end;

Procedure TFormularVis.TiskZapati;
var DT:TDateTime;
    R,M,D,H,Mi,S,MS:word;
begin
 DT:=JDToDateTime(MainForm.JDMin); DecodeDate(DT,R,M,D); DecodeTime(DT,H,Mi,S,MS);
 with P do begin
   Font.Name:='Arial CE';
   Font.Size:=10;
   Font.Style:=[fsBold];
   Align:=alignCenter;
   Move(moveLB,0,-2200);
   TabStop:=PosY-GetLineHeight;
   Shift(0,500);
   TableBegin(alignLeft,100,25);
   TableAddDiv(50);
   TableAddCol(1500,alignCenter);
   TableAddDiv(50);
   TableAddCol(1500,alignCenter);
   TableAddDiv(50);
   TableAddCol(1500,alignCenter);
   TableAddDiv(50);
   TableAddCol(2500,alignCenter);
   TableAddDiv(50);
   TableAddCol(1500,alignCenter);
   TableAddDiv(50);
   TableAddCol(1500,alignCenter);
   TableAddDiv(50);
   TableLine(50);
   TableRowBegin;
   TableCellBegin(3);
   Write('Okamik minima jasnosti');
   TableCellBegin(1);
   Write('Poet odh.');
   TableLine(50);
   TableRowBegin;
   TableCellBegin;
   Write('den');
   TableCellBegin;
   Write('msc');
   TableCellBegin;
   Write('rok');
   TableCellBegin;
   Write('as [UT]');
   TableCellBegin;
   Write('celk.');
   TableCellBegin;
   Write('sest.');
   TableLine(50);
   Font.Size:=12;
   TableRowBegin(700);
   TableCellBegin;
   Shift(0,100); Write(Format('%.2d',[D]));
   TableCellBegin;
   Shift(0,100); Write(Format('%.2d',[M]));
   TableCellBegin;
   Shift(0,100); Write(Format('%.4d',[R]));
   TableCellBegin;
   Shift(0,100); Write(Format('%.2dh %.2dmin',[H,Mi]));
   TableCellBegin;
   Shift(0,100); Write(MainForm.Pocet.Text);
   TableCellBegin;
   Shift(0,100); Write(MainForm.Sestupnych.Text);
   TableLine(50);
   TableEnd;
 end;
end;

Procedure TFormularVis.TiskZahlaviTabulky;
begin
 with P do begin
   if Assigned(P.Table) then begin
     P.TableLine(50);
     P.TableEnd;
   end;
   Font.Name:='Arial CE';
   P.Font.Size:=10;
   P.Font.Style:=[];
   Move(moveLT,0,TabStart);
   TableBegin(alignLeft,100,10);
   TableAddDiv(50);
   TableAddCol(700,alignCenter);
   TableAddDiv(50);
   TableAddCol(1600,alignCenter);
   TableAddDiv(50);
   TableAddCol(1600,alignCenter);
   TableAddDiv(50);
   TableAddCol(1600,alignCenter);
   TableAddDiv(50);
   TableAddCol(3000,alignCenter);
   TableAddDiv(50);
   TableLine(50);
   Font.Style:=[fsBold];
   TableRowBegin;
   TableCellBegin;
   Write('.');
   TableCellBegin;
   Write('as [UT]');
   TableCellBegin;
   Write('Odhad');
   TableCellBegin;
   Write('Odh. st.');
   TableCellBegin;
   Write('Poznmka');
   Font.Style:=[];
   TableLine(50);
 end;
end;

Procedure TFormularVis.TiskTabulkySrovHv;
var i:integer;
begin
 with P do begin
   Font.Name:='Arial CE';
   Font.Size:=10;
   Font.Style:=[];
   Move(moveLT,0,TabStart);
   TableBegin(alignRight,100,10);
   TableAddDiv(50);
   TableAddCol(1600,alignCenter);
   TableAddDiv(50);
   TableAddCol(1600,alignCenter);
   TableAddDiv(50);
   TableAddCol(3000,alignCenter);
   TableAddDiv(50);
   TableLine(50);
   Font.Style:=[fsBold];
   TableRowBegin;
   TableCellBegin;
   Write('Srov. hv.');
   TableCellBegin;
   Write('Odh. st.');
   TableCellBegin;
   Write('Poznmka');
   Font.Style:=[];
   TableLine(50);
   with MainForm.Comps do begin
     for i:=1 to RowCount-1 do if (Length(Cells[0,i])>0)and(Length(Cells[1,i])>0) then begin
       { Pro kazdy radek tabulky srovnavacich hvezd }
       P.TableRowBegin;
       P.TableCellBegin;
       P.Write(Cells[0,i]);
       P.TableCellBegin;
       P.Write(CheckDec(Cells[1,i]));
       P.TableCellBegin;
       P.Write(Cells[2,i]);
       P.TableRowEnd;
     end;
   end;
   TableLine(50);
   TableEnd;
   Align:=alignLeft;
   Shift(10200,500);
   Font.Style:=[fsBold];
   Write('MAPKA: ');
   Font.Style:=[];
   Writeln(MainForm.MAPKA.Text);
 end;
end;

Procedure TFormularVis.Print;
var i:integer;
var DT:TDateTime;
    R,M,D,H,Mi,S,MS:word;
begin
 { *** OKRAJE A NASTAVENI *** }
 P.Header.Visible:=false;
 P.Footer.Visible:=false;
 P.DocClear;
 P.TopMargin:=1500;
 P.BottomMargin:=1500;
 P.LeftMargin:=2000;
 P.RightMargin:=2000;
 P.DocBegin;
 { *** TABULKA *** }
 P.Font.Name:='Arial CE';
 P.Move(moveLT,0,0);
 TiskZahlavi(P);
 TiskZapati(P);
 TiskTabulkySrovHv(P);
 TiskZahlaviTabulky(P);
 with MainForm.Grid do begin
   for i:=1 to RowCount-1 do if (Length(Cells[1,i])>0)and(Length(Cells[2,i])>0) then begin
     { Pro kazdy radek }
     if P.PosY>=Self.TabStop then begin
       P.TableLine(50);
       P.TableEnd;
       P.NewPage;
       TiskZahlavi(P);
       TiskZapati(P);
       TiskZahlaviTabulky(P);
     end;
     P.TableRowBegin;
     P.TableCellBegin;
     P.Write(IntToStr(i));
     P.TableCellBegin;
     P.Write(CheckDec(Cells[1,i]));
     P.TableCellBegin;
     P.Write(CheckDec(Cells[2,i]));
     P.TableCellBegin;
     P.Write(CheckDec(Cells[3,i]));
     P.TableCellBegin;
     P.Write(Cells[4,i]);
     P.TableRowEnd;
   end;
   P.TableLine(50);
   P.TableEnd;
 end;
 { *** GRAF *** }
 P.NewPage;
 P.Font.Name:='Arial CE';
 DT:=JDToDateTime(MainForm.JDMin); DecodeDate(DT,R,M,D); DecodeTime(DT,H,Mi,S,MS);
 with P do begin
   { Popis }
   Font.Size:=14;
   Font.Style:=[];
   Align:=alignLeft;
   Write(MainForm.Hvezda.Text);
   Align:=alignCenter;
   Write(MainForm.Pozorovatel.Text);
   Align:=alignRight;
   Write(DateTimeToNightDate(MainForm.Datum.Date));
   Writeln;
   Align:=alignCenter;
   Writeln(Format('min.: %2d. %2d. %4d %.2d:%.2d UT',[D,M,R,H,Mi]));
   Shift(0,500);
   { Graf }
   FrameBegin(P.FrameWidth,13000);
   TiskGrafu(P);
   FrameEnd;
 end;
end;

Procedure TFormularVis.TiskGrafu;
{ Vykresleni grafu na strane Urceni minima }
const MinDeltaY = 0.3;
      MinDeltaX = 2/24;
      XSteps :array [0..11] of double = (1, 12/24, 6/24, 3/24, 2/24, 1/24, 30/1440, 15/1440, 10/1440, 5/1440, 2/1440, 1/1440);
      VSteps :array [0..9] of double = (10, 5, 2, 1, 0.5, 0.2, 0.1, 0.05, 0.02, 0.01);
      VPrecs :array [0..9] of byte   = ( 0, 0, 0, 0,   1,   1,   1,    2,    2,    2);
var x,i,y,w,h,vprec,Wid:integer;
    j,dt,kt,qt,kv,qv,vmin,vmax,vstep,val,dv,tmin,tmax,time,tstep:double;
    Str:String;
    M:byte;
begin
 { Zjisti hranice grafu }
 W:=P.FrameWidth;
 H:=P.FrameHeight;
 with MainForm, MainForm.Grid do begin
   if JDMin<2400000 then JDMin:=int(DateTimeToJD(Datum.Date))+frac(JDMin);
   TMin:=JDMin; TMax:=JDMin;
   if not GetValue(1, VMin, M) then VMin:=0;
   if not GetValue(1, VMax, M) then VMax:=0;
   for i:=1 to RowCount-1 do if GetTime(i, Time) and GetValue(i, Val, M) then begin
     { Pro kazdy radek }
     if Time<TMin then TMin:=Time; if Time>TMax then TMax:=Time;
     if Val <VMin then VMin:=Val ; if Val >VMax then VMax:=Val;
   end;
 end;
 { Uprav meze x, aby graf nebyl mensi nez 2 hodiny }
 if (TMax-TMin)<MinDeltaX then begin
   DT:=(MinDeltaX-(TMax-TMin))/2;
   TMax:=TMax+DT;
   TMin:=TMin-DT;
 end;
 i:=Length(XSteps)-1;
 while (i>0)and(((TMax-TMin)/XSteps[i])>8) do i:=i-1;
 TStep:=XSteps[i];
 TMin:=floor(TMin/TStep)*TStep;
 TMax:=ceil(TMax/TStep)*TStep;
 { Uprav meze y, aby graf nebyl mensi nez MinDeltaY stupnu }
 if ((VMax-VMin)<MinDeltaY) then begin
   DV:=(MinDeltaY-(VMax-VMin))/2;
   VMax:=VMax+DV;
   VMin:=VMin-DV;
 end;
 i:=Length(VSteps)-1;
 while (i>0)and(((VMax-VMin)/VSteps[i])>10) do i:=i-1;
 VStep:=VSteps[i];
 VPrec:=VPrecs[i];
 VMin:=floor(VMin/VStep)*VStep;
 VMax:=ceil(VMax/VStep)*VStep;
 { Spocita mapovaci konstanty }
 KT:=(W-1300-800)/(TMax-TMin);
 QT:=1300-KT*TMin;
 KV:=(H-800-1200)/(VMax-VMin);
 QV:=800-KV*VMin;
 with P do begin
   { Nakresli osy }
   Move(moveLT,1300,0);
   P.VLine(H-1200,35);
   Move(moveLT,1300,H-1200);
   P.HLine(W-1300,35);
   { Popise osu x }
   Font.Size:=10;
   Font.Style:=[fsBold];
   move(moveRB,0,-GetLineHeight);
   Align:=alignRight; Write('as [UT]');
   Align:=alignLeft;
   Font.Name:='Arial CE';
   Font.Size:=10;
   Font.Style:=[];
   j:=floor(TMin/TStep)*TStep;
   while (j<TMax+TStep) do begin
     X:=round(KT*j+QT);
     Str:=FormatDateTime('hh:nn',JDToDateTime(j));
     Wid:=GetTextWidth(Str);
     if (X>=1300) and (X<=W-(Wid div 2)) then begin
       move(moveLT,X,H-1300); VLine(200,35);
       move(moveLT,X-(Wid div 2),H-950);
       Write(Str);
     end;
     j:=j+TStep;
   end;
   { Popise osu y }
   Font.Size:=10;
   Font.Style:=[fsBold];
   Align:=alignLeft;
   move(moveLT,0,0);
   Write('odh.st.');
   Font.Size:=10;
   Font.Style:=[];
   j:=floor(VMin/VStep)*VStep;
   while (j<VMax+VStep) do begin
     Y:=round(KV*j+QV);
     Str:=Format('%.*f',[VPrec,j]);
     Wid:=GetTextWidth(Str);
     if (Y>=500) and (Y<=H-1200) then begin
       move(moveLT,1200,Y); HLine(200,35);
       move(moveLT,1100-Wid,Y-(GetLineHeight div 2));
       Write(Str);
     end;
     j:=j+VStep;
   end;
   { Nakresli minimum }
   Pen.Color:=clBlack;
   Brush.Color:=clBlack;
   X:=round(KT*MainForm.JDMin+QT);
   move(moveLT,X,500); VLine(H-1200-500,15);
   { Nakresli tecky }
   for i:=1 to MainForm.Grid.RowCount-1 do begin
     if GetTime(i, Time) and GetValue(i, Val, M) then begin
       X:=round(KT*Time+QT);
       Y:=round(KV*Val+QV);
       if M=0 then begin       { tecka }
         move(moveLT,X,Y);
         Ellipse(150,150,true);
       end else
       if M=1 then begin
         move(moveLT,X-75,Y-75);
         Line(75,150,25);
         move(moveLT,X,Y+75);
         Line(75,-150,25);
       end else
       if M=2 then begin
         move(moveLT,X-75,Y+75);
         Line(75,-150,25);
         move(moveLT,X,Y-75);
         Line(75,150,25);
       end else
       if M=3 then begin
         move(moveLT,X-75,Y+75);
         Line(150,-150,25);
         move(moveLT,X+75,Y+75);
         Line(-150,-150,25);
       end;
     end;
   end;
 end;
end;

end.
