unit XMLStream;

(*
    Protokoly 2.0
    Copyright (C) 2002  Ing. David Motl

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

interface

Uses Classes, SysUtils, StrUtils, Math;

type
  TXMLParser  = class;
  TOnStartTag = procedure(ASender:TXMLParser; AnElement:String; AnAttributes:TStrings) of object;
  TOnEndTag   = procedure(ASender:TXMLParser; AnElement:String) of object;
  TOnPCData   = procedure(ASender:TXMLParser; AData:String) of object;

  TXMLParser = class(TMemoryStream)
   protected
    State:byte;
    FLast       : char;
    FFile       : TStream;
    FOnStartTag : TOnStartTag;
    FOnEndTag   : TOnEndTag;
    FOnPCData   : TOnPCData;
    FInLine, FElement,FAttribute,FValue,FPCData : String; // Buffers
    FreeOnClose : boolean;
    FAttributes : TStringList;
    Procedure   StartTag;
    Procedure   EndTag;
    Procedure   PCData;
    Function    GetCh:char;
    Function    EOF:boolean;
   public
    Line, Col :integer;
    CustomData:TObject;
    Constructor Create(AFile:TStream);
    Constructor CreateFromStream(AFile:TStream);
    Constructor CreateFromString(AStr:String);
    Destructor  Destroy; override;
    Procedure   Parse;
    Function    SearchAttr(AnElement,AnAttribute:String; out AValue:String):boolean;
    property    OnStartTag:TOnStartTag read FOnStartTag write FOnStartTag;
    property    OnEndTag:TOnEndTag read FOnEndTag write FOnEndTag;
    property    OnPCData:TOnPCData read FOnPCData write FOnPCData;
  end;

  TXMLErrorCode = (InvalidChar);

  EXMLError = class(Exception)
    Constructor Create(ASender:TXMLParser; ACode:TXMLErrorCode; AChar:char);
  end;

  Procedure   XMLStartTag(F:TStream; AnElement:String; Empty:boolean);
  Procedure   XMLBeginTag(F:TStream; AnElement:String);
  Procedure   XMLDoneTag(F:TStream; Empty:boolean);
  Procedure   XMLAttribute(F:TStream; AName,AValue:String); overload;
  Procedure   XMLAttribute(F:TStream; AName:String; AValue:integer); overload;
  Procedure   XMLAttribute(F:TStream; AName:String; AValue:boolean); overload;
  Procedure   XMLEndTag(F:TStream; AnElement:String);
  Procedure   XMLPCData(F:TStream; AData:String);
  Procedure   XMLPCLine(F:TStream; AData:String);
  Procedure   XMLSimpleData(F:TStream; AnElement:String; AData:String);
  Procedure   XMLSimpleTag(F:TStream; AnElement:String);
  Procedure   XMLHeader(F:TStream);

  { Cteni hodnot ze atributu }
  Function    XMLString(S:TStrings; AName:String; ADefault:String):String;
  Function    XMLInteger(S:TStrings; AName:String; ADefault:integer):integer;
  Function    XMLBoolean(S:TStrings; AName:String; ADefault:boolean):boolean;
  Function    XMLDouble (S:TStrings; AName:String; ADefault:double):double;

  Function    StreamToString(Str:TMemoryStream):String;
  Function    CheckDocType(XML,Typ:String):boolean;

implementation

Uses Utils;

const
 W1250ToISO : array[#128..#191] of byte = (
  $20, $20, $20, $20, $20, $20, $20, $20, $20, $20, $A9, $20, $A6, $AB, $AE, $AC,
  $20, $20, $20, $20, $20, $20, $AD, $20, $20, $20, $B9, $20, $B6, $BB, $BE, $BC,
  $A0, $B7, $A2, $A3, $A4, $A1, $20, $A7, $A8, $20, $AA, $20, $20, $20, $20, $AF,
  $B0, $20, $B2, $B3, $B4, $20, $20, $20, $B8, $B1, $BA, $20, $A5, $BD, $B5, $BF);

 ISOToW1250 : array[#160..#191] of byte = (
  $A0, $A5, $A2, $A3, $A4, $BC, $8C, $A7, $A8, $8A, $AA, $8D, $8F, $96, $8E, $AF,
  $B0, $B9, $B2, $B3, $B4, $BE, $9C, $A1, $B8, $9A, $BA, $9D, $9F, $BD, $9E, $BF);


const MaxBufSize = 8192;  // Zatim nic

      // Vyhrazene znaky
const StartCh    = '<';
      EndCh      = '>';
      EndSig     = '/';
      QMark      = '?'; 
      StartVal   = '"';
      EndVal     = '"';
      StartInline= '&';
      EndInline  = ';';
      EqualSig   = '=';
      IDchars    = ['A'..'Z','a'..'z','0'..'9','_'];
      WS         = [#0..#32];
      CRLF:word  = $0A0D;

procedure WinToISO(const P: pointer; pocet: integer); overload;
var i: integer;
begin
 for i:= 0 to Pocet - 1 do if (PByteArray(P)[i] >= 128) and (PByteArray(P)[i]<192) then
   PByteArray(P)[i] := ord(W1250ToISO[char(PByteArray(P)[i])]);
end;

Function WinToISO(S:String):String; overload;
begin
 WinToISO(@S[1],Length(S));
 Result:=S;
end;

Function InlineToStr(Seq:String):String;
begin
 if Seq='LT' then Result:='<' else
 if Seq='GT' then Result:='>' else
 if Seq='AMP' then Result:='&' else
 if Seq='APOS' then Result:='''' else
 if Seq='QUOT' then Result:='"' else
 Result:=''; 
end;

Function StrToInline(AData:String):String;
var i:integer;
begin
 i:=1;
 while (i<=Length(Adata)) do begin 
   if Adata[i]='<' then begin Delete(Adata,i,1); Insert(StartInline+'lt'+EndInline,AData,i); i:=i+3; end else
   if Adata[i]='>' then begin Delete(Adata,i,1); Insert(StartInline+'gt'+EndInline,AData,i); i:=i+3; end else
   if Adata[i]='&' then begin Delete(Adata,i,1); Insert(StartInline+'amp'+EndInline,AData,i); i:=i+4; end else
   if Adata[i]='''' then begin Delete(Adata,i,1); Insert(StartInline+'apos'+EndInline,AData,i); i:=i+5; end else
   if Adata[i]='"' then begin Delete(Adata,i,1); Insert(StartInline+'quot'+EndInline,AData,i); i:=i+5; end;
   i:=i+1;
 end;
 Result:=Adata;
end;

Constructor TXMLParser.Create;
begin
 inherited Create;
 CustomData:=NIL; 
 FAttributes:=TStringList.Create;
 LoadFromStream(AFile); FFile:=AFile;
 Line:=1; Col:=1; FLast:=#0;
 State:=0; Position:=0;
 FreeOnClose:=true;
end;

Constructor TXMLParser.CreateFromStream;
begin
 Create(AFile);
 FreeOnClose:=false;
end;

Constructor TXMLParser.CreateFromString;
begin
 inherited Create;
 FAttributes:=TStringList.Create;
 Size:=Length(AStr);
 WriteBuffer(AStr[1],Length(AStr));
 FFile:=NIL;
 Line:=1; Col:=1; FLast:=#0;
 State:=0; Position:=0; 
end;

Destructor TXMLParser.Destroy;
begin
 if FreeOnClose then FFile.Free;
 FAttributes:=NIL;
 inherited;
end;

Procedure TXMLParser.Parse;
var ch:char;
begin
 Position:=0; 
 FPCData:=''; Felement:=''; FAttribute:=''; FValue:=''; FInline:=''; FAttributes.Clear;
 while (not EOF) do begin
   ch:=GetCh;
   case State of
     0: if ch=StartCh then begin if FPCData<>'' then PCData; FPCData:=''; FAttributes.Clear; State:=1 end
        else if ch=StartInline then State:=17                // Zacatek inline sekvence
        else if (ch>=#32) and(ch<#128) then FPCData:=FPCData+ch     // Znak textu
        else if (ch>=#128)and(ch<#160) then FPCData:=FPCData+#20     // Znak textu
        else if (ch>=#160)and(ch<#192) then FPCData:=FPCData+char(ISOToW1250[ch])  // Znak textu
        else if (ch>=#192) then FPCData:=FPCData+ch;                // Znak textu
     1: if ch=EndSig then State:=14       // </
        else if ch=QMark then State:=19   // <?
        else if ch in WS then State:=13   // mezery
        else if ch in IDchars then begin Felement:=upcase(ch); State:=2; end // Prvni znak elementu
        else raise EXMLError.Create(Self,InvalidChar,ch);
     2: if ch in IDchars then begin Felement:=FElement+upcase(ch); end
        else if ch=EndSig then State:=11                    // />
        else if ch=EndCh then begin StartTag; State:=0; end // >
        else if ch in WS then State:=3        // Mezera za elementem
        else raise EXMLError.Create(Self,InvalidChar,ch);
     3: if ch in IDchars then begin FAttribute:=upcase(ch); State:=4; end // Prvni znak atributu
        else if ch=EndSig then State:=11                    // />
        else if ch=EndCh then begin StartTag; State:=0; end // >
        else if ch in WS then                 // Mezera za elementem
        else raise EXMLError.Create(Self,InvalidChar,ch);
     4: if ch in IDchars then begin FAttribute:=FAttribute+upcase(ch); end // Atribut (nazev)
        else if ch=EqualSig then State:=6                    // =
        else if ch in WS then State:=5                       // Mezera za atributem
        else raise EXMLError.Create(Self,InvalidChar,ch);
     5: if ch=EqualSig then State:=6                         // =
        else if ch in WS then                                // Mezera za atributem
        else raise EXMLError.Create(Self,InvalidChar,ch);
     6: if ch=StartVal then begin FValue:=''; State:=7; end  // Zacatek retezce
        else if ch in WS then                                // Mezera za atributem
        else raise EXMLError.Create(Self,InvalidChar,ch);
     7: if ch=EndVal then begin FAttributes.Add(FAttribute+'='+FValue);
                     FAttribute:=''; FValue:=''; State:=10; end
        else if ch=StartInline then State:=8
        else if (ch>=#160) and (ch<#192) then FValue:=FValue+char(ISOToW1250[ch])
        else if (ch>=#128) and (ch<#160) then FValue:=FValue+#20
        else FValue:=FValue+ch;
     8: if ch in IDchars then begin FInline:=upcase(ch); State:=9; end // Inline sekvence
        else raise EXMLError.Create(Self,InvalidChar,ch);
     9: if ch in IDchars then begin FInline:=FInLine+upcase(ch); end // Atribut (nazev)
        else if ch=EndInline then begin FValue:=FValue+InlineToStr(FInline); State:=7; end
        else raise EXMLError.Create(Self,InvalidChar,ch);
    10: if ch in WS then State:=3
        else if ch=EndCh then begin StartTag; State:=0; end
        else if ch=EndSig then State:=11
        else raise EXMLError.Create(Self,InvalidChar,ch);
    11: if ch=EndCh then begin StartTag; EndTag; State:=0; end
        else raise EXMLError.Create(Self,InvalidChar,ch);
    13: if ch in WS then                                // Mezera za <
        else if ch in IDchars then begin Felement:=upcase(ch); State:=2; end // Prvni znak elementu
        else raise EXMLError.Create(Self,InvalidChar,ch);
    14: if ch in WS then                 // Mezera pred elementem
        else if ch in IDchars then begin Felement:=upcase(ch); State:=15; end // Prvni znak elementu
        else raise EXMLError.Create(Self,InvalidChar,ch);
    15: if ch in IDchars then begin Felement:=FElement+upcase(ch); end
        else if ch=EndCh then begin EndTag; State:=0; end // >
        else if ch in WS then State:=16        // Mezera za elementem
        else raise EXMLError.Create(Self,InvalidChar,ch);
    16: if ch=EndCh then begin EndTag; State:=0; end      // >
        else if ch in WS then                 // Mezera za elementem
        else raise EXMLError.Create(Self,InvalidChar,ch);
    17: if ch in IDchars then begin FInline:=upcase(ch); State:=18; end // Inline sekvence
        else raise EXMLError.Create(Self,InvalidChar,ch);
    18: if ch in IDchars then begin FInline:=FInLine+upcase(ch); end    // Sekvence
        else if ch=EndInline then begin FPCData:=FPCData+InlineToStr(FInline); State:=0; end
        else raise EXMLError.Create(Self,InvalidChar,ch);
    19: if ch=QMark then begin State:=20; end;
    20: if ch=EndCh then begin State:=0; end;
   end;
  end;
end;

Procedure TXMLParser.StartTag;
begin
 if Assigned(FOnStartTag) then FOnStartTag(Self,FElement,FAttributes);
end;

Procedure TXMLParser.EndTag;
begin
 if Assigned(FOnEndTag) then FOnEndTag(Self,FElement);
end;

Procedure TXMLParser.PCData;
begin
 if Assigned(FOnPCData) then FOnPCData(Self,FPCData);
end;

Function TXMLParser.GetCh;
begin
 ReadBuffer(Result,1); 
 if Result=#13 then begin Line:=Line+1; Col:=1; end           // CR
 else if (Result=#10)and(FLast<>#13) then begin Line:=Line+1; Col:=1; end // LF
 else Col:=Col+1;
 FLast:=Result;
end;

Function TXMLParser.EOF;
begin
 Result:=Position>=Size;
end;

Procedure XMLBeginTag;
var S:String;
begin
 S:=StartCh+lowercase(AnElement);
 F.WriteBuffer(S[1],Length(S));
end;

Procedure XMLStartTag;
var S:String;
begin
 S:=StartCh+lowercase(AnElement);
 if Empty then S:=S+EndSig+EndCh+#13#10
          else S:=S+EndCh+#13#10;
 F.WriteBuffer(S[1],Length(S));
end;

Procedure XMLDoneTag;
var S:String;
begin
 if Empty then S:=EndSig+EndCh+#13#10
          else S:=EndCh+#13#10;
 F.WriteBuffer(S[1],Length(S));
end;

Procedure   XMLAttribute(F:TStream; AName,AValue:String); overload;
var S:String;
begin
 S:=' '+AName+'="'+StrToInline(WinToISO(AValue))+'"';
 F.WriteBuffer(S[1],Length(S));
end;

Procedure   XMLAttribute(F:TStream; AName:String; AValue:integer); overload;
var S:String;
begin
 S:=' '+AName+'="'+IntToStr(AValue)+'"';
 F.WriteBuffer(S[1],Length(S));
end;

Procedure   XMLAttribute(F:TStream; AName:String; AValue:boolean); overload;
var S:String;
begin
 S:=' '+AName+'="'+BoolToStr(AValue)+'"';
 F.WriteBuffer(S[1],Length(S));
end;

Procedure XMLEndTag;
var S:String;
begin
 S:=StartCh+EndSig+lowercase(AnElement)+EndCh+#13#10;
 F.WriteBuffer(S[1],Length(S));
end;

Procedure XMLPCData;
var S:String;
begin
 S:=StrToInline(WinToISO(AData));
 F.WriteBuffer(S[1],Length(S));
end;

Procedure XMLPCLine;
var S:String;
begin
 S:=StrToInline(WinToISO(AData))+#13#10;
 F.WriteBuffer(S[1],Length(S));
end;

Procedure XMLSimpleData;
var S:String;
begin
 S:=StartCh+lowercase(AnElement)+EndCh
   +StrToInline(WinToISO(AData))
   +StartCh+EndSig+lowercase(AnElement)+EndCh+#13#10;
 F.WriteBuffer(S[1],Length(S));
end;

Procedure XMLSimpleTag;
var S:String;
begin
 S:=StartCh+lowercase(AnElement)+EndSig+EndCh;
 F.WriteBuffer(S[1],Length(S));
end;

Function TXMLParser.SearchAttr;
var ch:char;
begin
 Position:=0; FElement:=''; FAttribute:=''; FInline:='';
 while (not EOF) do begin
   ch:=GetCh;
   case State of
     0: if ch=StartCh then begin if FPCData<>'' then PCData; FPCData:=''; FAttributes.Clear; State:=1 end
        else if ch=StartInline then State:=17                // Zacatek inline sekvence
        else if (ch>=#32) and(ch<#128) then FPCData:=FPCData+ch     // Znak textu
        else if (ch>=#128)and(ch<#160) then FPCData:=FPCData+#20     // Znak textu
        else if (ch>=#160)and(ch<#192) then FPCData:=FPCData+char(ISOToW1250[ch])  // Znak textu
        else if (ch>=#192) then FPCData:=FPCData+ch;                // Znak textu
     1: if ch=EndSig then State:=14       // </
        else if ch=QMark then State:=19   // <?
        else if ch in WS then State:=13   // mezery
        else if ch in IDchars then begin Felement:=upcase(ch); State:=2; end // Prvni znak elementu
        else raise EXMLError.Create(Self,InvalidChar,ch);
     2: if ch in IDchars then begin Felement:=FElement+upcase(ch); end
        else if ch=EndSig then State:=11                    // />
        else if ch=EndCh then begin StartTag; State:=0; end // >
        else if ch in WS then State:=3        // Mezera za elementem
        else raise EXMLError.Create(Self,InvalidChar,ch);
     3: if ch in IDchars then begin FAttribute:=upcase(ch); State:=4; end // Prvni znak atributu
        else if ch=EndSig then State:=11                    // />
        else if ch=EndCh then begin StartTag; State:=0; end // >
        else if ch in WS then                 // Mezera za elementem
        else raise EXMLError.Create(Self,InvalidChar,ch);
     4: if ch in IDchars then begin FAttribute:=FAttribute+upcase(ch); end // Atribut (nazev)
        else if ch=EqualSig then State:=6                    // =
        else if ch in WS then State:=5                       // Mezera za atributem
        else raise EXMLError.Create(Self,InvalidChar,ch);
     5: if ch=EqualSig then State:=6                         // =
        else if ch in WS then                                // Mezera za atributem
        else raise EXMLError.Create(Self,InvalidChar,ch);
     6: if ch=StartVal then begin FValue:=''; State:=7; end  // Zacatek retezce
        else if ch in WS then                                // Mezera za atributem
        else raise EXMLError.Create(Self,InvalidChar,ch);
     7: if ch=EndVal then begin
          if (Felement=AnElement) and (Fattribute=AnAttribute) then begin Result:=true; AValue:=FValue; exit; end;
          FAttribute:=''; FValue:=''; State:=10; end
        else if ch=StartInline then State:=8
        else if (ch>=#160) and (ch<#192) then FValue:=FValue+char(ISOToW1250[ch])
        else if (ch>=#128) and (ch<#160) then FValue:=FValue+#20
        else FValue:=FValue+ch;
     8: if ch in IDchars then begin FInline:=upcase(ch); State:=9; end // Inline sekvence
        else raise EXMLError.Create(Self,InvalidChar,ch);
     9: if ch in IDchars then begin FInline:=FInLine+upcase(ch); end // Atribut (nazev)
        else if ch=EndInline then begin FValue:=FValue+InlineToStr(FInline); State:=7; end
        else raise EXMLError.Create(Self,InvalidChar,ch);
    10: if ch in WS then State:=3
        else if ch=EndCh then begin StartTag; State:=0; end
        else if ch=EndSig then State:=11
        else raise EXMLError.Create(Self,InvalidChar,ch);
    11: if ch=EndCh then begin StartTag; EndTag; State:=0; end
        else raise EXMLError.Create(Self,InvalidChar,ch);
    13: if ch in WS then                                // Mezera za <
        else if ch in IDchars then begin Felement:=upcase(ch); State:=2; end // Prvni znak elementu
        else raise EXMLError.Create(Self,InvalidChar,ch);
    14: if ch in WS then                 // Mezera pred elementem
        else if ch in IDchars then begin Felement:=upcase(ch); State:=15; end // Prvni znak elementu
        else raise EXMLError.Create(Self,InvalidChar,ch);
    15: if ch in IDchars then begin Felement:=FElement+upcase(ch); end
        else if ch=EndCh then begin EndTag; State:=0; end // >
        else if ch in WS then State:=16        // Mezera za elementem
        else raise EXMLError.Create(Self,InvalidChar,ch);
    16: if ch=EndCh then begin EndTag; State:=0; end      // >
        else if ch in WS then                 // Mezera za elementem
        else raise EXMLError.Create(Self,InvalidChar,ch);
    17: if ch in IDchars then begin FInline:=upcase(ch); State:=18; end // Inline sekvence
        else raise EXMLError.Create(Self,InvalidChar,ch);
    18: if ch in IDchars then begin FInline:=FInLine+upcase(ch); end    // Sekvence
        else if ch=EndInline then begin FPCData:=FPCData+InlineToStr(FInline); State:=0; end
        else raise EXMLError.Create(Self,InvalidChar,ch);
    19: if ch=QMark then begin State:=20; end;
    20: if ch=EndCh then begin State:=0; end;
   end;
  end;
  AValue:=''; Result:=false; exit;
end;

Constructor EXMLError.Create;
begin
 inherited Create(IntToStr(ASender.Line)+':'+IntToStr(ASender.Col)+' Chybn znak "'+AChar+'"');
end;

Function StreamToString;
begin
 SetLength(Result,Str.Size);
 Move(Str.Memory^,Result[1],Str.Size);
end;

Function CheckDocType;
begin
 Result:=true;
end; 

Function XMLString;
begin
 Result:=ifThen(S.IndexOfName(AName)>=0,S.Values[AName],ADefault);
end;

Function XMLInteger;
begin
 Result:=ifThen(S.IndexOfName(AName)>=0,StrToIntDef(S.Values[AName],ADefault),ADefault);
end;

Function XMLBoolean;
begin
 Result:=ifThen(S.IndexOfName(AName)>=0,StrToBoolDef(S.Values[AName],ADefault),ADefault);
end;

Function XMLDouble;
begin
 Result:=ifThen(S.IndexOfName(AName)>=0,StrToFloatDef(S.Values[AName],ADefault),ADefault);
end;

Procedure XMLHeader;
const S:String='<?xml version="1.0" encoding="ISO_8859-2" ?>'#13#10;
begin
 F.WriteBuffer(S[1],Length(S));
end;

end.


